#========================================================================================
# package      DE_EPAGES::Ebay::UI::EbayTransaction
# state        private
# base         DE_EPAGES::Presentation::UI::Object
#========================================================================================
package DE_EPAGES::Ebay::UI::EbayTransaction;
use base qw( DE_EPAGES::Presentation::UI::Object );

use strict;
use DE_EPAGES::Core::API::Error qw ( GetError ExistsError );
use DE_EPAGES::Core::API::Log qw (LogError);
use DE_EPAGES::Core::API::String qw (QuoteXML);
use DE_EPAGES::Ebay::API::Ebay qw (UpdateTransactionList);
use DateTime;

#========================================================================================
# function     SendFeedback
# state        private
#----------------------------------------------------------------------------------------
# syntax       SendFeedback( $Servlet );
#----------------------------------------------------------------------------------------
# description  sends customer feedback to ebay
#========================================================================================
sub SendFeedback {
    my $self    = shift;
    my ($Servlet) = @_;

    my $MasterObject = $Servlet->object;
    my $Form = $Servlet->form;

    my $Shop = $MasterObject->getSite();
    my $hValues = $Form->form($MasterObject->getSite(), 'EbayTransactionFeedback');
    my @ListObjects = map { $Shop->loadObject( $_->{'ListObjectID'}, 'EbayTransaction' ) }
                      @{$hValues->{'ListObjectIDs'}};
    $Servlet->testPermission($_, 'Save') foreach @ListObjects;

    my $text = $Form->formValue('FeedbackText');

    foreach my $transaction (@ListObjects) {
        eval {
            $transaction->leaveFeedback('Positive', QuoteXML($text));
        };

        if (ExistsError()) {
            my $Error = GetError();
            my $msg = $Error->{'Vars'}->{'SOAPMessage'};
            $msg = join(', ', map {$_->{'LongMessage'}} @{$Error->{'Vars'}->{'EbayErrors'}}) if not $msg;

            $Form->addFormError({
                'Name' => 'EbayRequestNotSuccessful',
                'Reason' => 'EbayRequestNotSuccessful',
                'Transaction' => $transaction->tleHash,
                'Message' => $msg,
            });
        } else {
            $transaction->set({
                'FeedbackType' => 1,
                'FeedbackText' => $text,
            });
        }
    }
    $Form->executeFormError;

    return;
}

#========================================================================================
# function     SendFulfillMail
# state        private
#----------------------------------------------------------------------------------------
# syntax       SendFulfillMail( $Servlet );
#----------------------------------------------------------------------------------------
# description  sends customer email with directions on how to complete his order
#========================================================================================
sub SendFulfillMail {
    my $self    = shift;
    my ($Servlet) = @_;

    my $MasterObject = $Servlet->object;
    my $Form = $Servlet->form;

    my $Shop = $MasterObject->getSite();
    my $hValues = $Form->form($MasterObject, 'ListedObjects');
    my @ListObjects = map { $Shop->loadObject( $_->{'ListObjectID'}, 'EbayTransaction' ) }
                      @{$hValues->{'ListObjectIDs'}};
    $Servlet->testPermission($_, 'Save') foreach @ListObjects;

    foreach my $transaction (@ListObjects) {
        $transaction->sendFulfillmentMail() unless $transaction->get('CustomerOrder');
    }

    return;
}

#========================================================================================
# function     CompleteOrder
# state        private
#----------------------------------------------------------------------------------------
# syntax       CompleteOrder( $Servlet );
#----------------------------------------------------------------------------------------
# description  set ebay transaction as completed so that it can be imported as order
#========================================================================================
sub CompleteOrder {
    my $self    = shift;
    my ($Servlet) = @_;

    my $Object = $Servlet->object;
    my $Form = $Servlet->form;

    my $Shop = $Object->getSite();
    my $hValues = $Form->form($Object, 'ListedObjects');
    my @ListObjects = map { $Shop->loadObject( $_->{'ListObjectID'}, 'EbayTransaction' ) }
                      @{$hValues->{'ListObjectIDs'}};
    $Servlet->testPermission($_, 'Save') foreach @ListObjects;

    my %UpdateShopEnvList = ();
    # for safety reasons: check last five minutes
    my $now = DateTime->now;
    my $duration = DateTime::Duration->new( minutes  => 5 );

    foreach my $transaction (@ListObjects) {
        next if ($transaction->get('CustomerOrder'));
        $transaction->set({'IsEbayComplete' => 0});
        my $Auction = $transaction->get('EbayAuction');
        return unless (defined $Auction);

        eval {
            $transaction->completeOrder();
            my $ShopEnv = $Auction->get('EbayShopEnvironment');
            $UpdateShopEnvList{$ShopEnv->id} = $ShopEnv;
        };
        if (ExistsError) {
            my $Error = GetError();
            LogError($Error) if ($Error->code eq 'NoResults_Attribute');
            $Form->addFormError({
                'Reason'        => 'CreateOrderNotSuccessful',
                'Name'          => $Error->code,
                'Account'       => $transaction->get('EbayUserName'),
            });
        }
    }

    foreach my $ShopEnv (values %UpdateShopEnvList){
        my $RemoteEbay = $ShopEnv->remoteEbay();
        eval {
            my $aTransactions = $RemoteEbay->getSellerTransactions($now-$duration);
            UpdateTransactionList($ShopEnv, $aTransactions, 0);
        };
        if (ExistsError()) {
            my $Error = GetError();
            LogError($Error);
            $Form->addFormError({
                'Reason'        => 'CreateOrderNotSuccessful',
                'Name'          => $Error->code,
                'Account'       => $ShopEnv->get('EbayUserName'),
            });
        }
    }
    $Form->executeFormError();

    return;
}

1;
