#==============================================================================
# §package      DE_EPAGES::FacebookBusiness::UI::Shop
# §base         DE_EPAGES::Order::UI::Shop
# §state        private
#==============================================================================
package DE_EPAGES::FacebookBusiness::UI::Shop;
use base DE_EPAGES::Presentation::UI::Object;

use strict;
use DateTime;
use DE_EPAGES::Core::API::Error          qw( ExistsError GetError );
use DE_EPAGES::Core::API::PerlTools      qw( Compare GetCurrentDateTime GetLocalTimeZone ElementOf );
use DE_EPAGES::FacebookBusiness::API::Constants qw ( OAUTH_SCOPES );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );
use DE_EPAGES::FacebookBusiness::API::Object::FacebookServerClient;

use MIME::Base64;
use feature 'switch';

sub UnitySaveFacebookPixel {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;

    # save PixelID
    my $PixelID = $Servlet->form->value('pixel_id');

    if ($Shop->existsChild('FacebookBusinessShopEnvironment')) {
        # save pixel id or delete...
        my $hSave;
        $hSave->{'PixelID'} = $PixelID ? $PixelID : undef;
        $Shop->child('FacebookBusinessShopEnvironment')->set($hSave);
    }

    return;

}

sub UnityDisconnectFacebookBusiness {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;

    if ($Shop->existsChild('FacebookBusinessShopEnvironment')) {
        $Shop->child('FacebookBusinessShopEnvironment')->delete;
    }

}

sub UnityViewFacebookBusiness {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;

    # trigger hook to allow the adding of context sensitive apps
    TriggerHook('UI_Shop_UnityViewFacebookBusiness', { 'Servlet' => $Servlet });

    # get the Auth URL from RemoteFBServer....
    my $FacebookServerClient = DE_EPAGES::FacebookBusiness::API::Object::FacebookServerClient->new($Shop);
    my $RedirectURL = $Servlet->baseLineContentLocation . '?ViewAction=UnityMBO-ViewFacebookBusiness&ChangeAction=UnityMBO-SaveFacebookBusinessUserToken';

    # build the complete url on the FBServer by a call....
    # and test if the epages fb server can be connected
    my $hResult;
    eval {
        $hResult = $FacebookServerClient->call('getAuthURL', [$RedirectURL]);
    };
    if(ExistsError()) {
        $Servlet->vars('GET_OAUT_URL_ERROR',1);
        return;
    }

    # write URL to the Servlet....
    $Servlet->vars('OAuthURL', $hResult->{'Result'});

    # we have already an environment for the user
    if ($Shop->existsChild('FacebookBusinessShopEnvironment')) {

        # test if we have a token
        my $UserAccessToken = $Shop->child('FacebookBusinessShopEnvironment')->get('UserAccessToken');
        unless ($UserAccessToken){
            $Servlet->vars('UserAccessTokenIsValid', 0);
            return $self->SUPER::View($Servlet);
        }

        # debug token for getting userid and expiry date
        # test if we have a valid token with debug token
        my $hTokenDebugResponse = $self->_getInfoToken($FacebookServerClient, $UserAccessToken);

        my $UserAccessTokenIsValid  = $hTokenDebugResponse->{'data'}->{'is_valid'} eq JSON::XS::true ? 1 : 0;
        $Servlet->vars('UserAccessTokenIsValid', $UserAccessTokenIsValid);
        return $self->SUPER::View($Servlet) unless $UserAccessTokenIsValid;

        my $UserAccessTokenIsExpired;
        my $DataAccessExpiresAt = $hTokenDebugResponse->{'data'}->{'data_access_expires_at'};
        # only if we have an expiry infomation
        if($DataAccessExpiresAt){
            my $UserAccessTokenExpiresInDate = DateTime->from_epoch('epoch'     => $DataAccessExpiresAt, 
                                                                    'time_zone' => GetLocalTimeZone());
            $UserAccessTokenIsExpired = ((defined $UserAccessTokenExpiresInDate) and 
                                        (Compare('DateTime',GetCurrentDateTime(),$UserAccessTokenExpiresInDate) >= 0)) ? 1 : 0;
        }
    
        $Servlet->vars('UserAccessTokenExpired', $UserAccessTokenIsExpired);
        return $self->SUPER::View($Servlet) if $UserAccessTokenIsExpired;

        my $aMissingScopes = $self->_getMissingScopes($hTokenDebugResponse);
        my $ScopeIsMissing = scalar(@$aMissingScopes) ? 1 : 0;
        
        $Servlet->vars('UserAccessTokenScopeIsMissing', $ScopeIsMissing );
        $Servlet->vars('UserAccessTokenScopesMissing', [map{ {'Scope' => $_} } @$aMissingScopes] );

        # return if we have missing scopes
        return $self->SUPER::View($Servlet) if $ScopeIsMissing;

        # get info about the FB user by ID       
        my $UserID = $hTokenDebugResponse->{'data'}->{'user_id'};
        my $hUserInfo = $self->_getInfoUserByUserID($FacebookServerClient, $UserAccessToken, $UserID);
        $Servlet->vars('UserInfo', $hUserInfo);

        my $PixelID = $Shop->child('FacebookBusinessShopEnvironment')->get('PixelID');
        if($PixelID){
            # we have a saved pixel id. get info for this id
            my $hPixelInfo = $self->_getAdsPixelInfo($FacebookServerClient, $PixelID );
            $Servlet->vars('PixelInfo', $hPixelInfo );
        } 
        # we have not an PixelID saved, so we get the info from the user account...
        # get info if there are som BMs and elements available
        my $hBMInfo = $self->_getBusinessManagersByUser($FacebookServerClient, $UserAccessToken, $UserID);
        my $hStateInfo =  $FacebookServerClient->createStateInfo($hBMInfo);

        foreach my $PixelEntry (@{$hStateInfo->{'PX'}}){
            # add the pixel detail info to the $hStateInfo
            my $hPixelInfo = $self->_getAdsPixelInfo($FacebookServerClient, $PixelEntry->{'id'} );
            $PixelEntry->{'name'} = $hPixelInfo->{'name'};
        }
        $Servlet->vars('Infostate', $hStateInfo);
    }

    return $self->SUPER::View($Servlet);
}

sub UnitySaveFacebookBusinessUserToken {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;
    my $FBServerParams = $Servlet->params();

    # test if we have error elements in uri params
    $Servlet->form->executeFormError({
        'Name'        => 'FB_ERROR_AUTHCODE',
        'FBName'      => $FBServerParams->{'error'},
        'Reason'      => $FBServerParams->{'error_reason'} // 'FB_ERROR_AUTHCODE',
        'Description' => $FBServerParams->{'error_description'},
    }) if $FBServerParams->{'error'};

    # test if we have a browser auth code
    $Servlet->form->executeFormError({
        'Name'      => 'FB_ERROR_AUTHCODE',
        'Reason'    => 'FB_ERROR_AUTHCODE',
    }) unless $FBServerParams->{'code'};

    eval {

        my $FacebookServerClient = DE_EPAGES::FacebookBusiness::API::Object::FacebookServerClient->new($Shop);
        my $hResponse = $FacebookServerClient->call('getOAuthUserTokenByAuthCode', [$FBServerParams->{'code'}]);

        my $UserAccessToken = $hResponse->{'access_token'};
        return unless $UserAccessToken;

        # debug token for getting userid and expiry date
        my $hTokenDebugResponse = $self->_getInfoToken($FacebookServerClient, $UserAccessToken);
        my $DataAccessExpiresAt = $hTokenDebugResponse->{'data'}->{'data_access_expires_at'};
        my $UserID = $hTokenDebugResponse->{'data'}->{'user_id'};
        my $UserAccessTokenExpiresInDate = DateTime->from_epoch('epoch'     => $DataAccessExpiresAt,
                                                            'time_zone' => GetLocalTimeZone());
        
        # set Token, UserID, exiry date and test if token is valid
        my $FacebookBusinessShopEnvironment = $FacebookServerClient->shopEnvironment();
        $FacebookBusinessShopEnvironment->set({
                                            'UserAccessToken'           => $UserAccessToken,
                                            'UserAccessTokenExpiryDate' => $UserAccessTokenExpiresInDate,
                                            'UserID'                    => $UserID,
                                            });
    };
    if(ExistsError()) {
        my $Error = GetError();
        my $FBErrorMessage = $Error->vars()->{'DecodedMessage'}->{'error'}->{'message'};

        $Servlet->form->executeFormError({
            'Name'        => 'FB_ERROR_AUTHCODE',
            'Reason'      => 'FB_ERROR_AUTHCODE',
            'Message'     => $FBErrorMessage,
        });
    }

    return;
}

sub _getInfoToken {
    my $self = shift;
    my ($FacebookServerClient, $UserAccessToken) = @_;

    my $hResponse;
    eval {
        $hResponse = $FacebookServerClient->call('debugOAuthUserToken', [$UserAccessToken]);
    };
    if(ExistsError()) {
        my $Error = GetError();
        my $FBErrorMessage = $Error->vars()->{'DecodedMessage'}->{'error'}->{'message'};
        $self->servlet()->form->executeFormError({
            'Name'        => 'FB_ERROR_DEBUG_TOKEN',
            'Reason'      => 'FB_ERROR_DEBUG_TOKEN',
            'Message'     => $FBErrorMessage,
        });
    }

    return $hResponse;

}

sub _getMissingScopes {
    my $self = shift;
    my ($hTokenInfo) = @_;

    my $aScopes = $hTokenInfo->{'data'}->{'scopes'};
    my $aScopesMissing = [];
    my @ConstScopes = split(/,/,OAUTH_SCOPES);
    foreach my $Scope (@ConstScopes){
        unless (ElementOf($Scope, @$aScopes)) {
            push @$aScopesMissing, $Scope; 
        }
    }

    return $aScopesMissing;

}

sub _getBusinessManagersByUser {
    my $self = shift;
    my ($FacebookServerClient, $UserAccessToken, $UserID) = @_;

    my $hResponse;
    eval {
        $hResponse = $FacebookServerClient->call('getBusinessManagersByUser', [$UserAccessToken, $UserID]);
    };
    if(ExistsError()) {
        my $Error = GetError();
        my $FBErrorMessage = $Error->vars()->{'DecodedMessage'}->{'error'}->{'message'};
        $self->servlet()->form->executeFormError({
            'Name'        => 'FB_ERROR_GET_BM',
            'Reason'      => 'FB_ERROR_GET_BM',
            'Message'     => $FBErrorMessage,
        });
    }

    return $hResponse;
}

sub _getInfoUserByUserID {
    my $self = shift;   
    my ($FacebookServerClient, $UserAccessToken, $UserID) = @_;

    my $hResponse;
    eval {
        $hResponse = $FacebookServerClient->call('getInfoUserByUserID', [$UserAccessToken, $UserID]);
    };
    if(ExistsError()) {
        my $Error = GetError();
        my $FBErrorMessage = $Error->vars()->{'DecodedMessage'}->{'error'}->{'message'};
        $self->servlet()->form->executeFormError({
            'Name'        => 'FB_ERROR_GET_INFOUSER',
            'Reason'      => 'FB_ERROR_GET_INFOUSER',
            'Message'     => $FBErrorMessage,
        });
    }

    return $hResponse;

}

sub _getAdsPixelInfo {
    my $self = shift;
    my ($FacebookServerClient, $PixelID) = @_;

    my $hInfoUser = $FacebookServerClient->shopEnvironment->get([qw(UserAccessToken)]);

    my $hResponse;
    eval {
        $hResponse = $FacebookServerClient->call('getAdsPixelInfo', [$hInfoUser->{'UserAccessToken'}, $PixelID]);
    };

    if(ExistsError()) {
        my $Error = GetError();
        $self->servlet()->form->executeFormError({
            'Name'        => 'FB_ERROR_GET_ADSPIXEL_INFO',
            'Reason'      => 'FB_ERROR_GET_ADSPIXEL_INFO',
            'Message'     => $Error->message,
            'UserTitle'   => $Error->vars()->{'Response'}->{'error'}->{'error_user_title'},
            'UserMsg'     => $Error->vars()->{'Response'}->{'error'}->{'error_user_msg'},
            'message'     => $Error->vars()->{'Response'}->{'error'}->{'message'},
            'code'        => $Error->vars()->{'Response'}->{'error'}->{'code'},
            'fbtrace_id'  => $Error->vars()->{'Response'}->{'error'}->{'fbtrace_id'},
            'type'        => $Error->vars()->{'Response'}->{'error'}->{'type'},
        });
    }
    return $hResponse;

}

1;
