#========================================================================================
# package       DE_EPAGES::Klarna::UI::BasketOrder
# base          DE_EPAGES::Presentation::UI::Object
# state         public
# description   Basket and order UI methods.
#========================================================================================
package DE_EPAGES::Klarna::UI::BasketOrder;
use base DE_EPAGES::Presentation::UI::Object;

use strict;
use feature qw(switch);
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( LogDebug LogError );
use DE_EPAGES::Core::API::Object::DateTimeFormatter;
use DE_EPAGES::Core::API::PerlTools qw ( GetCurrentDateTime isValid trim );
use DE_EPAGES::Core::API::TestParameter qw ( TestObject );
use DE_EPAGES::Database::API::Connection qw ( GetCurrentDBHandle );
use DE_EPAGES::ExternalPayment::API::Log qw ( LogPayment );
use DE_EPAGES::Klarna::API::Constants qw ( %PROTOCOL KLARNA_STATUS_PENDING );
use DE_EPAGES::Klarna::API::KlarnaClient;
use DE_EPAGES::Klarna::API::KlarnaOperations::Activate;
use DE_EPAGES::Klarna::API::KlarnaOperations::CancelReservation;
use DE_EPAGES::Klarna::API::KlarnaOperations::ReserveAmount;
use DE_EPAGES::Klarna::API::KlarnaOperations::CheckOrderStatus;
use DE_EPAGES::Klarna::API::KlarnaOperations::UpdateReservation;
use DE_EPAGES::Klarna::API::KlarnaOrderStatus qw ( UpdateOrderStatus );
use DE_EPAGES::Object::API::Table::Country qw ( GetInfoCountry );
use DE_EPAGES::Order::API::Object::BasketOrderProcess;
use DE_EPAGES::Order::API::Order qw ( Basket2Order );

# PaymentErrors: key: Klarna status code, value: short name of error
# if the status code is one of the keys of this hash show payment page again
# otherwise show error page
my %PaymentErrors = (
    '2201' => 'pno',
    '2202' => 'invalid_pno',
    '2203' => 'pno_not_real_person',
    '2204' => 'dead',
    '2301' => 'no_such_person',
    '2302' => 'customer_missing',
    '2303' => 'no_such_customer',
    '2304' => 'unknown_customer',
);

use constant UNITY_FAILURE_ERROR => 'Failed';

#========================================================================================
# function      _cancelOrder
# state         private
#----------------------------------------------------------------------------------------
# syntax        _cancelOrder($Servlet);
#----------------------------------------------------------------------------------------
# description   Set order status to Cancelled if DoPaymentOnBasket is not set for the
#                payment method
#----------------------------------------------------------------------------------------
# input         $Order | the CustomerOrder object | object
# input         $PaymentMethod | the payment method object | object
# return        -
#========================================================================================
sub _cancelOrder {
    my $self = shift;
    my ( $Order, $PaymentMethod ) = @_;

    $Order->set( { 'CancelledOn' => GetCurrentDBHandle()->currentDateTime } ) unless ( $PaymentMethod->get('DoPaymentOnBasket') );
    return;
}

#========================================================================================
# function     _createOrder
# state        _private
#----------------------------------------------------------------------------------------
# syntax       _createOrder( $Object );
#----------------------------------------------------------------------------------------
# description  create the Order if it is still a Basket
#----------------------------------------------------------------------------------------
# input        $BasketOrOrder | basket or an order | object
# return       $Order | an order | object
#========================================================================================
sub _createOrder {
    my $self = shift;
    my ($BasketOrOrder) = @_;
    my $Order;
    if ( $BasketOrOrder->instanceOf('Basket') ) {
        $Order = $self->_Basket2Order($BasketOrOrder);
    }
    elsif ( $BasketOrOrder->instanceOf('CustomerOrder') ) {
        $Order = $BasketOrOrder;
    }
    else {
        Error( 'UnknownInstanceType', $BasketOrOrder );
    }
    return $Order;
}

#========================================================================================
# function     _sendOrderToKlarna
# state        _private
#----------------------------------------------------------------------------------------
# syntax       _sendOrderToKlarna( $hParams );
#----------------------------------------------------------------------------------------
# description  send the current Order to Klarna.
#----------------------------------------------------------------------------------------
# input        $hParams | Params needed for call | refhash
# return       $hResponse | Hash with response | refhash
#========================================================================================
sub _sendOrderToKlarna {
    my $self = shift;
    my ($hParams) = @_;

    my $hResponse;
    my $KlarnaLineItemPayment = $hParams->{'BasketOrder'}->get('LineItemContainer')->get('Payment');
    if ( $self->_isAdvancedIntegration($KlarnaLineItemPayment) ) {
        $hResponse = $self->ReserveAmount($hParams);
    }
    else {
        $hResponse = $self->AddTransaction($hParams);
    }

    return $hResponse;
}

#========================================================================================
# function     ReserveAmount
# state        public
#----------------------------------------------------------------------------------------
# syntax       ReserveAmount( $hParams );
#----------------------------------------------------------------------------------------
# description  call reserve Amount of the Advanced Integration
#----------------------------------------------------------------------------------------
# input        $hParams | Params needed for call | refhash
# return       $hResponse | Hash with response | refhash
#========================================================================================
sub ReserveAmount {
    my $self = shift;
    my ($hParams) = @_;

    my $ReserveAmountOperation = DE_EPAGES::Klarna::API::KlarnaOperations::ReserveAmount->new($hParams);
    my $hResponse              = $ReserveAmountOperation->call();

    return $hResponse;
}

#========================================================================================
# function     AddTransaction
# state        public
#----------------------------------------------------------------------------------------
# syntax       AddTransaction( $hParams );
#----------------------------------------------------------------------------------------
# description  call addTransaction of the standard Integration
#----------------------------------------------------------------------------------------
# input        $hParams | Params needed for call | refhash
# return       $hResponse | Hash with response | refhash
#========================================================================================
sub AddTransaction {
    my $self = shift;
    my ($hParams) = @_;

    my $Client    = DE_EPAGES::Klarna::API::KlarnaClient->new();
    my $hResponse = $Client->AddTransaction($hParams);

    return $hResponse;
}

#========================================================================================
# function     _saveResponseToLineItemPayment
# state        private
#----------------------------------------------------------------------------------------
# syntax       _saveResponseToLineItemPayment( $Input1 );
#----------------------------------------------------------------------------------------
# description  Save the Response to the LineItemPayment
#----------------------------------------------------------------------------------------
# input        $hResponse | LineitemPayment | ref_hash
# input        $LineItemPaymentKlarna | LineitemPayment | object
#========================================================================================
sub _saveResponseToLineItemPayment {
    my $self = shift;
    my ( $hResponse, $LineItemPaymentKlarna ) = @_;

    if ( $self->_isAdvancedIntegration($LineItemPaymentKlarna) ) {
        my $ReservationNumber = $hResponse->{'Message'}[0]->{'string'};
        $LineItemPaymentKlarna->set( {
                'ReservationNumber' => $ReservationNumber // '',
                'TransTime'         => $self->_GetCurrentDateTime(),
                'Message'           => undef,
                'Protocol'          => $PROTOCOL{'Advanced'},
            }
        );
    }
    else {
        my $InvoiceNumber = $hResponse->{'Message'}[0]->{'string'};
        $LineItemPaymentKlarna->set( {
                'Kr_InvoiceNo' => $InvoiceNumber // '',
                'TransTime'    => $self->_GetCurrentDateTime(),
                'Message'      => undef,
                'Protocol'     => $PROTOCOL{'Standard'},
            }
        );
    }
    return;
}

#========================================================================================
# function      SendDataToKlarna
# state         public
#----------------------------------------------------------------------------------------
# syntax        $self->SendDataToKlarna($Servlet);
#----------------------------------------------------------------------------------------
# description   Function sends payment data to Klarna.
#----------------------------------------------------------------------------------------
# input         $Servlet | current servlet (contains form data) | object
#========================================================================================
sub SendDataToKlarna {
    my $self                = shift;
    my ($Servlet)           = @_;
    my $BasketOrOrder       = $Servlet->object;
    my $Form                = $Servlet->form;

    my $LineItemContainer   = $BasketOrOrder->get('LineItemContainer');
    my $LineItemPayment     = $LineItemContainer->get('Payment');
    my $LineItemPaymentType = $LineItemPayment->get('PaymentType');
    my $Shop                = $BasketOrOrder->getSite();

    LogDebug SendDataToKlarna => $LineItemPaymentType->alias;
    return unless ( defined($LineItemPayment) );
    return
      unless ( $LineItemPaymentType->alias eq 'KlarnaInvoice'
        || $LineItemPaymentType->alias eq 'KlarnaHirePurchase'
        || $LineItemPaymentType->alias eq 'KlarnaFinancing' );

    my $PaymentMethod = $LineItemPayment->get('PaymentMethod');
    return unless ( defined($PaymentMethod) );

    my $BillingAddress = $BasketOrOrder->get('BillingAddress');
    my $CellPhone      = _validateCellPhone($Form);

    $LineItemPayment->set( { 'Kr_CellPhone' => $CellPhone } );

    my $Street     = $Form->value('Kl_Street') // $BillingAddress->get('Street');
    my $HouseNo    = $Form->value('Kl_HouseNo');
    my $SecurityNo = $Form->value('Kr_SSN');
    my $Birthdate  = $Form->value('Kl_Birthdate');
    my $HouseNoExt = $Form->value('Kl_HouseNoExt');

    if ( $Form->value('Kl_IsFromGermany') || $Form->value('Kl_IsFromNetherlands') ) {

        $Servlet->vars( 'Kl_IsFromGermany',     1 ) if $Form->value('Kl_IsFromGermany');
        $Servlet->vars( 'Kl_IsFromNetherlands', 1 ) if $Form->value('Kl_IsFromNetherlands');

        _validateStreet( $Form, $Street );
        _validateHouseNo( $Form, $HouseNo );

        _validateGender( $Form, $BillingAddress );
    }

    if ( !isValid($Birthdate) && $BillingAddress->get('Birthday') ) {

        # use birthday from billing address
        my $Formatter = DE_EPAGES::Core::API::Object::DateTimeFormatter->new( locale => $Servlet->locale );
        $Birthdate = $Formatter->format_datetime( $BillingAddress->get('Birthday'), 'date', 'short' );
    }

    my $BillingCountry = ( $BillingAddress->get('Country') // $Shop->get('Address')->get('Country') )->{'Code2'};
    my $BirthDateTime;
    if ( $BillingCountry ~~ [qw(DE NL AT)] ) {
        $BirthDateTime = _validateBirthdate( $Form, $Birthdate, $Servlet->locale );
    }
    else {
        $BirthDateTime = _parseDate( $Birthdate, $Servlet->locale ) if ( defined($Birthdate) );
    }

    $Birthdate = _formatBirthdate( $BirthDateTime, $BillingCountry ) if ( defined($BirthDateTime) );
    _validateSSN( $Form, $SecurityNo, $BillingCountry ) if ( $BillingCountry ~~ [qw(SE FI DK NO)] );

    # Do checks
    $Form->form( $BasketOrOrder, 'KlarnaCustomerData' );
    $Form->executeFormError();    # is this necessary?

    my $OrderNo;
    if ( $BasketOrOrder->instanceOf('CustomerOrder') ) {
        $OrderNo = $BasketOrOrder->alias;
        $Servlet->vars( 'OrderNumber', $OrderNo );
    }

    my $RemoteAddr;
    if ( defined( $Servlet->params->{'REMOTE_ADDR'} ) ) {
        $RemoteAddr = $Servlet->params->{'REMOTE_ADDR'};
    }
    $SecurityNo = defined $Birthdate ? $Birthdate : $SecurityNo;
    my $hParams = {
        'BasketOrder'    => $BasketOrOrder,
        'PaymentMethod'  => $PaymentMethod,
        'SecurityNumber' => $SecurityNo,
        'RemoteAddr'     => $RemoteAddr,
        'Street'         => $Street,
        'HouseNo'        => $HouseNo,
        'HouseNoExt'     => $HouseNoExt,
    };

    my $hResponse;
    eval { $hResponse = $self->_sendOrderToKlarna($hParams); };
    if ( ExistsError() ) {
        my $Error = GetError();

        LogPayment( $LineItemPaymentType->alias, 'Error in AddTransaction for payment line item ' . $LineItemPayment->get('GUID'), $Error->message );

        # EPG-26795: set order status to Denied if Klarna denies payment in checkout
        if ( $BasketOrOrder->instanceOf('CustomerOrder') ) {
            $self->_cancelOrder( $BasketOrOrder, $PaymentMethod );
        }

        if ($Shop->isUnity()) {
            $LineItemPayment->set( { 'PaymentErrorCode' => UNITY_FAILURE_ERROR } );
        } else {
            $Servlet->vars('ViewObjectID', $Shop->id);
            $Servlet->vars('ViewAction', 'ViewKlarnaCancel');
            $Servlet->vars('PaymentError', 1);
            $Servlet->vars('PaymentErrorMsg', $Error->message());
            $Servlet->vars('KrPaymentMethodObject', $LineItemPayment->tleHash);
            $Servlet->vars('PaymentObject', $LineItemPayment->container->parent->tleHash);
        }
        return;
    }
    LogPayment( $LineItemPaymentType->alias, 'AddTransaction response for payment line item ' . $LineItemPayment->get('GUID'), $hResponse );

    if ( uc( $hResponse->{'StatusCode'} ) eq 'OK' ) {
        $self->_saveResponseToLineItemPayment( $hResponse, $LineItemPayment );

        my $Order;
        eval { $Order = $self->_createOrder($BasketOrOrder); };
        if ( ExistsError() ) {
            my $Error = GetError();
            LogPayment( $LineItemPaymentType->alias, 'Order creation failed', $Error );
            Error($Error);
        }

        my $InvoiceStatus = $hResponse->{'Message'}[1]->{'int'} // 1;
        if ( defined $Order ) {
            my $OrderStatusField = ( $InvoiceStatus == KLARNA_STATUS_PENDING ) ? 'PendingOn' : 'PaidOn';
            $Order->set( { $OrderStatusField => GetCurrentDBHandle()->currentDateTime } );
            $Servlet->vars( 'ViewObjectID', $Order->id() );
            $Servlet->vars( 'ViewAction',   'View' );
            $Servlet->registerSession( $Order->get('User'), 0 );
            if ($Shop->isUnity()) {
                $Servlet->vars('ViewAction', 'ViewMultiCheckoutCustomerOrder');
            }
        }

    }
    else {
        ## no critic qw(ProhibitMagicNumbers)
        $LineItemPayment->set( {
                'Message' => length( $hResponse->{'Message'} ) > 255 ? substr( $hResponse->{'Message'}, 0, 255 ) : $hResponse->{'Message'},
                'ReturnCode' => $hResponse->{'StatusCode'}
            }
        );
        ## use critic
        if ( exists( $PaymentErrors{ $hResponse->{'StatusCode'} } ) ) {
            if ($Shop->isUnity()) {
                $LineItemPayment->set( { 'PaymentErrorCode' => UNITY_FAILURE_ERROR } );
            } else {
                $Form->executeFormError({
                    'Name'       => $hResponse->{'Message'},
                    'Reason'     => $PaymentErrors{ $hResponse->{'StatusCode'} },
                    'Message'    => $hResponse->{'Message'},
                    'StatusCode' => $hResponse->{'StatusCode'},
                    'ViewAction' => $Form->value('ViewAction'),
                    'ViewObjectID' => $BasketOrOrder->id
                });
            }
        }
        else {
            if ($Shop->isUnity()) {
                $LineItemPayment->set( { 'PaymentErrorCode' => UNITY_FAILURE_ERROR } );
                $Servlet->vars( 'ViewObjectID', $BasketOrOrder->id() );
                $Servlet->vars('ViewAction', 'ViewMultiCheckoutCustomerOrder');
            } else {
                $Servlet->vars( 'ViewObjectID',          $Shop->id );
                $Servlet->vars( 'ViewAction',            'ViewKlarnaCancel' );
                $Servlet->vars( 'PaymentError',          1 );
                $Servlet->vars( 'PaymentErrorMsg',       $hResponse->{'Message'} );
                $Servlet->vars( 'StatusCode',            $hResponse->{'StatusCode'} );
                $Servlet->vars( 'KrPaymentMethodObject', $LineItemPayment->tleHash );
                $Servlet->vars( 'PaymentObject',         $LineItemPayment->container->parent->tleHash );
            }

            # EPG-26795: set order status to Denied if Klarna denies payment in checkout
            $self->_cancelOrder( $BasketOrOrder, $PaymentMethod ) if ( $BasketOrOrder->instanceOf('CustomerOrder') );
        }
    }

    return;
}

# -- private helper functions for SendDataToKlarna
sub _validateCellPhone {
    my ($Form) = @_;

    my $CellPhone;
    if ( $Form->value('Kr_CellPhone') ne 'skipped' ) {
        $CellPhone = $Form->value('Kr_CellPhone');
        if ( !isValid($CellPhone) ) {
            $Form->addFormError( {
                    'Name'       => 'Kr_CellPhone',
                    'Reason'     => 'STRING_EMPTY',
                    'ViewAction' => $Form->value('ViewAction'),
                }
            );
        }
    }

    return $CellPhone;
}

sub _validateStreet {
    my ( $Form, $Street ) = @_;

    if ( !isValid($Street) ) {
        $Form->addFormError( {
                'Name'       => 'Kl_Street',
                'Reason'     => 'STRING_EMPTY',
                'ViewAction' => $Form->value('ViewAction'),
            }
        );
    }

    return;
}

sub _validateHouseNo {
    my ( $Form, $HouseNo ) = @_;

    if ( !isValid($HouseNo) ) {
        $Form->addFormError( {
                'Name'       => 'Kl_HouseNo',
                'Reason'     => 'STRING_EMPTY',
                'ViewAction' => $Form->value('ViewAction'),
            }
        );
    }

    return;
}

sub _validateGender {
    my ( $Form, $BillingAddress ) = @_;

    my $Gender = $BillingAddress->get('Gender');    # use gender from billing address
    if ( !isValid($Gender) ) {
        $Gender = $Form->value('Kl_Gender');        # use gender from payment form
        if ( !isValid($Gender) ) {
            $Form->addFormError( {
                    'Name'       => 'Kl_Gender',
                    'Reason'     => 'STRING_EMPTY',
                    'ViewAction' => $Form->value('ViewAction'),
                }
            );
        }
        else {
            $BillingAddress->set( { 'Gender' => $Gender } );
        }
    }

    # Gender is no longer being added to the end of the SSN since Klarna API 4.1

    return;
}

sub _parseDate {
    my ( $DateStr, $Locale ) = @_;
    my $Formatter = DE_EPAGES::Core::API::Object::DateTimeFormatter->new( locale => $Locale );
    my $DateTime = $Formatter->parse_datetime( $DateStr, 'date' );

    return $DateTime;
}

sub _formatBirthdate {
    my ( $Birthdate, $CountryID ) = @_;

    my $Formats = {
        'DE' => '%d%m%Y',
        'AT' => '%d%m%Y',
        'NL' => '%d%m%Y',
    };

    my $Format = $Formats->{$CountryID};
    return $Birthdate->strftime($Format) if defined($Format);

    return $Birthdate->strftime('%d%m%Y');
}

sub _validateBirthdate {
    my ( $Form, $Birthdate, $Locale ) = @_;

    return unless defined($Locale);

    my $DateTime;
    eval { $DateTime = _parseDate( $Birthdate, $Locale ); };

    if ( ExistsError() ) {
        $Form->addFormError( {
                'Name'       => 'Kl_Birthdate',
                'Reason'     => 'STRING_EMPTY',
                'ViewAction' => $Form->value('ViewAction'),
            }
        );
    }

    return $DateTime;
}

sub _validateSSN {
    my ( $Form, $SSN, $CountryID ) = @_;

    # Regular expressions taken from Klarna's API example
    # Description of fields' format taken from https://integration.klarna.com/en/api/standard-integration/functions/addtransaction/ are not clear,
    # but examples taken from https://integration.klarna.com/en/api/download-api/ help
    my $SSNFormats = {
        'SE' => '^[0-9]{6,6}(([0-9]{2,2}[-\+]{1,1}[0-9]{4,4})|([-\+]{1,1}[0-9]{4,4})|([0-9]{4,6}))$',
        'FI' => '(?i)^[0-9]{6,6}(([A\+-]{1,1}[0-9]{3,3}[0-9A-FHJK-NPR-Y]{1,1})|([0-9]{3,3}[0-9A-FHJK-NPR-Y]{1,1})|([0-9]{1,1}-{0,1}[0-9A-FHJK-NPR-Y]{1,1}))$',
        'DK' => '^[0-9]{8,8}([0-9]{2,2})?$',
        'NO' => '^[0-9]{6,6}((-[0-9]{5,5})|([0-9]{2,2}((-[0-9]{5,5})|([0-9]{1,1})|([0-9]{3,3})|([0-9]{5,5))))$'
    };

    my $RegExp = $SSNFormats->{$CountryID};
    return unless isValid($RegExp);

    if ( !isValid($SSN) || !( $SSN =~ $RegExp ) ) {
        $Form->addFormError( {
                'Name'       => 'Kr_SSN',
                'Reason'     => 'STRING_EMPTY',
                'ViewAction' => $Form->value('ViewAction'),
            }
        );
    }

    return;
}

#========================================================================================
# function      ViewPaymentForm
# state         public
#----------------------------------------------------------------------------------------
# syntax        $self->ViewPaymentForm($Servlet);
#----------------------------------------------------------------------------------------
# description   View action for payment form page.
#----------------------------------------------------------------------------------------
# input         $Servlet | current servlet (contains form data) | object
# return        -
#========================================================================================
sub ViewPaymentForm {
    my $self      = shift;
    my ($Servlet) = @_;
    my $Object    = $Servlet->object;

    my $Address = $Object->get('BillingAddress');
    return unless defined $Address;

    my $CountryID = $Address->get('CountryID');
    return unless defined $CountryID;

    my $hCountry    = GetInfoCountry($CountryID);
    my $CountryCode = $hCountry->{'Code2'};
    $Servlet->vars( 'Kl_IsFromGermany',     1 ) if $CountryCode eq 'DE';
    $Servlet->vars( 'Kl_IsFromNetherlands', 1 ) if $CountryCode eq 'NL';

    my $hStreet     = $Address->get( [qw( StreetOnly HouseNumber )] );
    my $Street      = $hStreet->{StreetOnly};
    my $Housenumber = $hStreet->{HouseNumber};
    $Servlet->vars( 'Kl_Street',  $Street );
    $Servlet->vars( 'Kl_HouseNo', $Housenumber );

    if ( $CountryCode eq 'NL' ) {

        #Get HousenumberExtension
        $Servlet->vars( 'Kl_IsFromNederlands', 1 );
        my $HousenumberExtension;
        if ( $Housenumber =~ m/(\d+)\s*(\D+)/ ) {
            $Housenumber          = $1;
            $HousenumberExtension = $2;
            $Servlet->vars( 'Kl_HouseNo',    ( trim($Housenumber) )[0] );
            $Servlet->vars( 'Kl_HouseNoExt', ( trim($HousenumberExtension) )[0] );
        }
    }

    my $CountryType = 0;
    if ( $CountryCode ~~ [qw(SE DK FI NO)] ) {
        $CountryType = 1;
        $Servlet->vars( 'RequiredGender',    0 );
        $Servlet->vars( 'RequiredBirthdate', 0 );
        $Servlet->vars( 'RequiredAddress',   0 );
    }
    else {
        $Servlet->vars( 'RequiredGender',    defined( $Address->get('Gender') )   ? 0 : 1 );
        $Servlet->vars( 'RequiredBirthdate', defined( $Address->get('Birthday') ) ? 0 : 1 );
        $Servlet->vars( 'RequiredAddress', ( defined($Street) && defined($Housenumber) ) ? 0 : 1 );
    }
    $Servlet->vars( 'BillingCountryType', $CountryType );

    my $PhoneAvailable
      = defined( $Address->get('Phone') )
      || defined( $Address->get('PhoneBusiness') )
      || defined( $Address->get('PhoneCell') )
      || defined( $Address->get('PhonePrivate') );
    $Servlet->vars( 'RequiredPhone', $PhoneAvailable ? 0 : 1 );
    return;
}

#========================================================================================
sub ViewMultiPaymentKlarna {
    my $self = shift;
    my ($Servlet) = @_;

    my $BasketOrderProcess = DE_EPAGES::Order::API::Object::BasketOrderProcess->new($Servlet);
    $Servlet->vars( 'BasketOrderProcess', $BasketOrderProcess->tleHash );

    $self->ViewPaymentForm($Servlet);
    return;
}

#========================================================================================
# function     CancelKlarnaInvoice
# state        public
#----------------------------------------------------------------------------------------
# syntax       CancelKlarnaInvoice( $Servlet );
#----------------------------------------------------------------------------------------
# description  Function that cancel a Klarna reservation
#----------------------------------------------------------------------------------------
# input        $Servlet | epages servlet | object
# return       $Servlet | Servlet object | object
#========================================================================================
sub CancelKlarnaInvoice {
    my $self              = shift;
    my ($Servlet)         = @_;
    my $Order            = $Servlet->object;
    my $LineItemContainer = $Order->get('LineItemContainer');
    my $hResult;
    eval {
        $hResult = $self->_cancelInvoice($LineItemContainer);
    };
    if (ExistsError()) {
        my $Error = GetError();
        # prepare for JavaScript Error message
        $hResult->{'StatusCode'} = $Error->code;
        $hResult->{'Message'}    = $Error->message;
        Error($Error) if ( $Error->code eq 'NativDie' );
        LogError($Error);
    }

    if ( uc( $hResult->{'StatusCode'} ) eq 'OK' ) {
        $Order->set( { 'PaidOn'      => undef } );
        $Order->set( { 'CancelledOn' => GetCurrentDBHandle()->currentDateTime } );
        $Servlet->vars( 'Kr_CancelErrorStatus', 'OK' );
    }
    else {
        $Servlet->vars( 'Kr_CancelErrorStatus',  $hResult->{'StatusCode'} );
        $Servlet->vars( 'Kr_CancelErrorMessage', $hResult->{'Message'} );
    }

    $Servlet->vars( 'ResponseContent', $hResult );

    return;
}

#----------------------------------------------------------------------------------------
sub _cancelInvoice {
    my $self = shift;
    my ($LineItemContainer) = @_;

    my $LineItemPaymentKlarna = $LineItemContainer->get('Payment');
    my $hResult;
    if ( $self->_isAdvancedIntegration($LineItemPaymentKlarna) ) {
        $hResult = $self->CancelInvoiceAdvancedIntegration($LineItemPaymentKlarna);
    }
    else {
        $hResult = $self->CancelInvoiceStandardIntegration($LineItemContainer);
    }
    return $hResult;
}

#========================================================================================
# function     CancelInvoiceAdvancedIntegration
# state        public
#----------------------------------------------------------------------------------------
# syntax       CancelInvoiceAdvancedIntegration( $LineItemPaymentKlarna );
#----------------------------------------------------------------------------------------
# description  Do the webservice call "cancel reservation" for advanced integration
#----------------------------------------------------------------------------------------
# input        $LineItemPaymentKlarna | LineItemContainer | object
# return       $hResult | hash with the result of the webservice call | refhash
#========================================================================================
sub CancelInvoiceAdvancedIntegration {
    my $self = shift;
    my ($LineItemPaymentKlarna) = @_;

    my $CancelReservation = DE_EPAGES::Klarna::API::KlarnaOperations::CancelReservation->new($LineItemPaymentKlarna);
    my $hResult = $CancelReservation->call();

    return $hResult;
}

#========================================================================================
# function     CancelInvoiceStandardIntegration
# state        public
#----------------------------------------------------------------------------------------
# syntax       CancelInvoiceStandardIntegration( $LineItemContainer );
#----------------------------------------------------------------------------------------
# description  Do the webservice call "cancel Invoice" for Standard integration
#----------------------------------------------------------------------------------------
# input        $LineItemContainer | LineItemContainer | object
# return       $hResult | hash with the result of the webservice call | refhash
#========================================================================================
sub CancelInvoiceStandardIntegration {
    my $self = shift;
    my ($LineItemContainer) = @_;

    my $Client = DE_EPAGES::Klarna::API::KlarnaClient->new();
    my $hResult = $Client->CancelInvoice($LineItemContainer);

    return $hResult;
}

#========================================================================================
# function     _setPaymentIntegrationType
# state        private
#----------------------------------------------------------------------------------------
# syntax       _setPaymentIntegrationType( $LineItemPaymentKlarna );
#----------------------------------------------------------------------------------------
# description  If the klarna payment LineItem is in Advanced intergation the class var is set
#----------------------------------------------------------------------------------------
# input        $LineItemPaymentKlarna | lineItemPayment from Klarna | object
# return       $IsAdvancedIntegration | 1 whether is advanced integration | integer
#========================================================================================
sub _isAdvancedIntegration {
    my $self                    = shift;
    my ($LineItemPaymentKlarna) = @_;

    my $IsAdvancedIntegration   = 0;
    my $Protocol                = $self->_getProtocolFromLineItem($LineItemPaymentKlarna);
    if ( $Protocol eq $PROTOCOL{'Advanced'} ) {
        $IsAdvancedIntegration = 1;
    }

    return $IsAdvancedIntegration;
}

#========================================================================================
# function     ActivateKlarnaInvoice
# state        public
#----------------------------------------------------------------------------------------
# syntax       ActivateKlarnaInvoice( $Servlet );
#----------------------------------------------------------------------------------------
# description  Description
#----------------------------------------------------------------------------------------
# input        $Servlet | epages servlet | object
# return       $Servlet | Empty return | undef
#========================================================================================
sub ActivateKlarnaInvoice {
    my $self              = shift;
    my ($Servlet)         = @_;
    my $Order             = $Servlet->object;
    my $LineItemContainer = $Order->get('LineItemContainer');
    my $hResult           = undef;
    eval {
        $hResult = $self->_ActivateInvoice($LineItemContainer);
    };
    if ( ExistsError() ) {
        my $Error = GetError();
        # prepare for JavaScript Error message
        $hResult->{'StatusCode'} = $Error->code;
        $hResult->{'Message'}    = $Error->message;
        Error($Error) if ( $Error->code eq 'NativDie' );
        LogError($Error);
    }
    if ( uc( $hResult->{'StatusCode'} ) eq 'OK' ) {
        $self->_StoreResultInOrder( $hResult, $Order );
        $Servlet->vars( 'Kr_ActivationErrorStatus', 'OK' );
        $Order->set({'PaidOn' => GetCurrentDateTime()});
        $Order->set({'PendingOn' => undef});
    }
    else {
        $Servlet->vars( 'Kr_ActivationErrorStatus',  $hResult->{'StatusCode'} );
        $Servlet->vars( 'Kr_ActivationErrorMessage', $hResult->{'Message'} );
    }

    $Servlet->vars( 'ResponseContent', $hResult );

    return;
}

#========================================================================================
# function     _ActivateInvoice
# state        private
#----------------------------------------------------------------------------------------
# syntax       _ActivateInvoice( $LineItemContainer );
#----------------------------------------------------------------------------------------
# description  calls the "activate Invoice" webservice of klarna
#----------------------------------------------------------------------------------------
# input        $LineItemContainer | a linitemContainer | object
# return       $hResult | the results in hash structure | refhash
#========================================================================================
sub _ActivateInvoice {
    my $self                 = shift;
    my ($LineItemContainer)  = @_;
    my $LineItemPaymentKlarna = $LineItemContainer->get('Payment');
    my $hResult;
    if ( $self->_isAdvancedIntegration($LineItemPaymentKlarna) ) {
        $hResult = $self->ActivateInvoiceAdvancedIntregration($LineItemPaymentKlarna);
    }
    else {
        $hResult = $self->ActivateInvoiceStandardIntegration($LineItemContainer);
    }
    return $hResult;
}

#========================================================================================
# function     _StoreResultInOrder
# state        private
#----------------------------------------------------------------------------------------
# syntax       _StoreResultInOrder( $hResult, $Order );
#----------------------------------------------------------------------------------------
# description  store the result of the Webservice request in the Order
#----------------------------------------------------------------------------------------
# input        $hResult | the results in hash structure | refhash
# input        $Order | a Order | object
#========================================================================================
sub _StoreResultInOrder {
    my $self = shift;
    my ( $hResult, $Order ) = @_;

    my $LineItemPaymentKlara = $Order->get('LineItemContainer')->get('Payment');

    # store in LineItemPaymentKlarna
    if ( $self->_isAdvancedIntegration($LineItemPaymentKlara) ) {
        $self->_storeResultForAdvancedIntegration( $hResult, $LineItemPaymentKlara );
    }
    else {
        $self->_storeResultForStandardIntegration( $hResult, $LineItemPaymentKlara );
    }

    # set in Order
    $Order->set( { 'KlarnaInvoiceActivated' => 1 } );

    return;
}

#========================================================================================
# function     _storeResultForAdvancedIntegration
# state        private
#----------------------------------------------------------------------------------------
# syntax       _storeResultForAdvancedIntegration( $hResult, $LineItemPaymentKlarna );
#----------------------------------------------------------------------------------------
# description  stores the result of the advanced webservice request in the LineItemPayment
#----------------------------------------------------------------------------------------
# input        $hResult | the results in hash structure | refhash
# input        $LineItemPaymentKlara | LineItemPaymentKlarna | object
#========================================================================================
sub _storeResultForAdvancedIntegration {
    my $self = shift;
    my ( $hResult, $LineItemPaymentKlarna ) = @_;
    my $aMessage = $hResult->{'Message'};

    if (isValid($aMessage) && isValid(@{$aMessage}[1])){
        my $InvoiceNumber = @{$aMessage}[1]->{'string'};
        if ( isValid($InvoiceNumber) ) {
            $LineItemPaymentKlarna->set( {
                    'Kr_InvoiceNo'     => $InvoiceNumber
                }
            );
        }
    }
    return;
}

#========================================================================================
# function     _storeResultForStandardIntegration
# state        private
#----------------------------------------------------------------------------------------
# syntax       _storeResultForStandardIntegration( $hResult, $LineItemPaymentKlarna );
#----------------------------------------------------------------------------------------
# description  stores the result of the Standard webservice request in the LineItemPayment
#----------------------------------------------------------------------------------------
# input        $hResult | the results in hash structure | refhash
# input        $LineItemPaymentKlara | LineItemPaymentKlarna | object
#========================================================================================
sub _storeResultForStandardIntegration {
    my $self = shift;
    my ( $hResult, $LineItemPaymentKlara ) = @_;

    my $InvoicePdfUrl = $hResult->{'Message'}->{'string'};
    if ( isValid($InvoicePdfUrl) ) {
        $LineItemPaymentKlara->set( { 'Kr_InvoicePdfUrl' => $InvoicePdfUrl } );
    }

    return;
}

#========================================================================================
# function     ActivateInvoiceAdvancedIntregration
# state        public
#----------------------------------------------------------------------------------------
# syntax       ActivateInvoiceAdvancedIntregration( $LineItemContainer );
#----------------------------------------------------------------------------------------
# description  do the webservice call "activate Invoice" for advanced integration
#----------------------------------------------------------------------------------------
# input        $LineItemContainer | LineItemContainer | object
# return       $hResult | hash with the result of the webservice call | refhash
#========================================================================================
sub ActivateInvoiceAdvancedIntregration {
    my $self = shift;
    my ($LineItemPaymentKlarna) = @_;

    my $activate = DE_EPAGES::Klarna::API::KlarnaOperations::Activate->new( $LineItemPaymentKlarna );
    my $hResult = $activate->call();

    return $hResult;
}

#========================================================================================
# function     ActivateInvoiceStandardIntegration
# state        public
#----------------------------------------------------------------------------------------
# syntax       ActivateInvoiceStandardIntegration( $LineItemContainer );
#----------------------------------------------------------------------------------------
# description  do the webservice call "activate Invoice" for Standard integration
#----------------------------------------------------------------------------------------
# input        $LineItemContainer | LineItemContainer | object
# return       $hResult | hash with the result of the webservice call | refhash
#========================================================================================
sub ActivateInvoiceStandardIntegration {
    my $self = shift;
    my ($LineItemContainer) = @_;

    my $Client  = DE_EPAGES::Klarna::API::KlarnaClient->new();
    my $hResult = $Client->ActivateInvoice($LineItemContainer);

    return $hResult;
}

#========================================================================================
# function        DoKlarnaOrderStatusCheck
# state           public
#----------------------------------------------------------------------------------------
# syntax          my $result = $self->DoKlarnaOrderStatusCheck($Servlet);
#----------------------------------------------------------------------------------------
# description     Request order status updates from Klarna and update the order flags
#                  accordingly. For use in the MBO
#----------------------------------------------------------------------------------------
# input           $Servlet | Servlet object | object
# return          $Servlet | Empty return | undef
#========================================================================================
sub DoKlarnaOrderStatusCheck {
    my $self = shift;
    my ($Servlet) = @_;
    my $Order = $Servlet->object;
    my $LineItemContainer = $Order->get('LineItemContainer');
    my $hResult;

    eval{
        $hResult = $self->_CheckKlarnaOrderStatus($LineItemContainer);
    };
    if ( ExistsError() ) {
        my $Error = GetError();
        # prepare for JavaScript Error message
        $hResult->{'StatusCode'} = $Error->code;
        $hResult->{'Message'}    = $Error->message;
        Error($Error) if ( $Error->code eq 'NativDie' );
        LogError($Error);
    } else {
        $Servlet->vars('Kr_CheckStatusStatus', $hResult->{'StatusCode'});
        $Servlet->vars('Kr_CheckStatusMessage', $hResult->{'Message'});
    }

    if ( uc( $hResult->{'StatusCode'} ) eq 'OK' ) {
        $Servlet->vars( 'Kr_ActivationErrorStatus', 'OK' );
    }

    $Servlet->vars( 'ResponseContent', $hResult );

    return;
}

#========================================================================================
# function     UpdateKlarnaReservation
# state        public
#----------------------------------------------------------------------------------------
# syntax       UpdateKlarnaReservation( $Servlet );
#----------------------------------------------------------------------------------------
# description  Trigger the Klarna update reservation webservice
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet object | object
# return       $Servlet | Empty return | undef
#========================================================================================
sub UpdateKlarnaReservation {
    my $self = shift;
    my ($Servlet) = @_;
    my $Order = $Servlet->object;
    my $LineItemContainer = $Order->get('LineItemContainer');
    my $LineItemPaymentKlarna = $LineItemContainer->get('Payment');
    my $hResult;

    eval{
        my $UpdateReservation = DE_EPAGES::Klarna::API::KlarnaOperations::UpdateReservation->new($LineItemContainer);
        $hResult = $UpdateReservation->call();
    };
    if ( ExistsError() ) {
        my $Error = GetError();
        # prepare for JavaScript Error message
        $hResult->{'StatusCode'} = $Error->code;
        $hResult->{'Message'}    = $Error->message;
        Error($Error) if ( $Error->code eq 'NativDie' );
        LogError($Error);
    } elsif ( uc( $hResult->{'StatusCode'} ) eq 'OK' ) {
        my $LineItemContainerGrandTotal = $LineItemContainer->get('GrandTotal');
        $LineItemPaymentKlarna->set( {
            'Amount' => $LineItemContainerGrandTotal,
            'IsReservationOutOfDate' => 0,
        });
    }

    $Servlet->vars( 'ResponseContent', $hResult );

    return;
}

#========================================================================================
# function     _getProtocolFromLineItem
# state        private
#----------------------------------------------------------------------------------------
# syntax       _getProtocolFromLineItem( $LineItemPaymentKlarna );
#----------------------------------------------------------------------------------------
# description  read the Protocol from Payment lineitem
#----------------------------------------------------------------------------------------
# input        $LineItemPaymentKlarna | lineItemPayment from Klarna | object
# return       $Protocol | Protocol of payment method | String
#========================================================================================
sub _getProtocolFromLineItem {
    my $self = shift;
    my ($LineItemPaymentKlarna) = @_;

    TestObject( 'LineItemPaymentKlarna', $LineItemPaymentKlarna, 'DE_EPAGES::Klarna::API::Object::LineItemPaymentKlarna' );
    my $Protocol = $LineItemPaymentKlarna->get('Protocol');

    return $Protocol;
}

#========================================================================================
# function     _CheckKlarnaOrderStatus
# state        private
#----------------------------------------------------------------------------------------
# syntax       _CheckKlarnaOrderStatus( $LineItemContainer );
#----------------------------------------------------------------------------------------
# description  calls the "check order status" webservice of klarna
#----------------------------------------------------------------------------------------
# input        $LineItemContainer | lineItemContainer from order | object
# return       $hResult | the results in hash structure | refhash
#========================================================================================
sub _CheckKlarnaOrderStatus {
    my $self = shift;
    my ($LineItemContainer)  = @_;
    my $LineItemPaymentKlarna = $LineItemContainer->get('Payment');
    my $Order = $LineItemContainer->parent();

    my $OrderStatus = DE_EPAGES::Klarna::API::KlarnaOperations::CheckOrderStatus->new( $LineItemPaymentKlarna );
    my $hResponse = $OrderStatus->call();

    my $hResult = UpdateOrderStatus($Order, $hResponse);

    return $hResult;
}

# === wrapp the function for testing ===
sub _GetCurrentDateTime {
    my $self = shift;
    return GetCurrentDateTime();
}


sub _Basket2Order {
    my $self = shift;
    my ($BasketOrOrder) = @_;
    return Basket2Order($BasketOrOrder);
}

1;
