#========================================================================================
# package      DE_EPAGES::Newsletter::API::Object::Newsletter
# state        public
# base         DE_EPAGES::Object::API::Object::Object
# description  object interface for Newsletter.
#========================================================================================
package DE_EPAGES::Newsletter::API::Object::Newsletter;
use base DE_EPAGES::Object::API::Object::Object;

use strict;
use Email::Address;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError Error );
use DE_EPAGES::Core::API::File qw ( ExistsFile );
use DE_EPAGES::Core::API::Html qw ( ReplaceURIWithURL );
use DE_EPAGES::Core::API::Log qw ( GetLog LogError );
use DE_EPAGES::Core::API::String qw ( SplitEMailAddresses );
use DE_EPAGES::Core::API::Warning qw ( Warning );
use DE_EPAGES::Core::API::PerlTools qw ( GetCurrentDateTime );
use DE_EPAGES::Core::API::TestParameter qw ( TestHash TestEMailAddress TestObject );
use DE_EPAGES::Mail::API::Mail;
use DE_EPAGES::MailType::API::Object::SmtpConnection;
use DE_EPAGES::Newsletter::API::Constants qw (NEWSLETTER_SENT MAILS_PER_RUN %NEWSLETTER_ERROR);
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject LoadClassByAlias );
use DE_EPAGES::Presentation::API::Object::ProgressMonitor;
use DE_EPAGES::Presentation::API::SiteTLEProcessor qw ( ReplaceTLEs );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );

#========================================================================================
# function     className
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ClassName = $Newsletter->className;
#----------------------------------------------------------------------------------------
# description  Returns the class name, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $ClassName | class name | String
#========================================================================================
sub className  { 'Newsletter' }

#========================================================================================
# function     dbPackage
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DbPackage = $Newsletter->dbPackage;
#----------------------------------------------------------------------------------------
# description  Returns the database package, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $DbPackage | database package name | String
#========================================================================================
sub dbPackage { __PACKAGE__ }

#========================================================================================
# function     insert
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Newsletter = DE_EPAGES::Newsletter::API::Object::Newsletter->insert($hInfo);
# example      my $Class = LoadClassByAlias('Newsletter');
#               my $Newsletter = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Insert this info as object, afterwards the hook 'OBJ_InsertNewsletter' is triggered.
#----------------------------------------------------------------------------------------
# input        $hInfo | attributes of new object | ref.hash
# return       $Newsletter | new object | object
# hook         OBJ_InsertNewsletter | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>NewsletterID  - Newsletter Identifier - int
#               </ul>
#========================================================================================
sub insert {
    my $class = shift;
    my ($hInfo) = @_;

    my $self = $class->SUPER::insert($hInfo);
    TriggerHook('OBJ_InsertNewsletter', {'Object' => $self, 'NewsletterID' => $self->id });
    delete $hInfo->{$_} foreach qw ( NewsletterCampaign IsHTML Name Subject SenderName SenderEMail ScheduleDate SentDate Status DeliveryInProgress ); ## no critic qw(ProhibitLongLines)
    return $self;
}

#========================================================================================
# function     delete
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Newsletter->delete;
#----------------------------------------------------------------------------------------
# description  Deletes this object, therefore the hook 'OBJ_DeleteNewsletter' is triggered.
#----------------------------------------------------------------------------------------
# hook         OBJ_DeleteNewsletter | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>NewsletterID  - Newsletter Identifier - int
#               </ul>
#========================================================================================
sub delete {
    my $self = shift;

    TriggerHook('OBJ_DeleteNewsletter', {'Object' => $self, 'NewsletterID' => $self->id });
    LoadClassByAlias('NewsletterAttachment')->deleteObjects('ByNewsletterID', [$self->id]);
    LoadClassByAlias('NewsletterRecipient')->deleteObjects('ByNewsletterID', [$self->id]);
    $self->SUPER::delete;
    return;
}

#========================================================================================
# function     set
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Newsletter->set($hValues, $LanguageID);
#----------------------------------------------------------------------------------------
# description  This function triggers hooks 'OBJ_BeforeUpdateNewsletter' and 'OBJ_AfterUpdateNewsletter'.
#----------------------------------------------------------------------------------------
# input        $hValues | attribute name => value hash | ref.hash
# input        $LanguageID | (optional; required for localized attributes) language id | int
#----------------------------------------------------------------------------------------
# hook         OBJ_BeforeUpdateNewsletter | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>NewsletterID  - Newsletter Identifier - int
#                 <li>Values        - new values for object - ref.hash
#                 <li>LanguageID    - values of this language will be updated - int
#               </ul>
# hook         OBJ_AfterUpdateNewsletter | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>NewsletterID  - Newsletter Identifier - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul>
#========================================================================================
sub set {
    my $self = shift;
    my ($hValues, $LanguageID) = @_;

    TriggerHook('OBJ_BeforeUpdateNewsletter', {'Object'=>$self, 'Values'=> $hValues, 'NewsletterID' => $self->id, 'LanguageID' => $LanguageID});
    $self->SUPER::set($hValues, $LanguageID);
    TriggerHook('OBJ_AfterUpdateNewsletter',  {'Object'=>$self, 'Values'=> $hValues, 'NewsletterID' => $self->id, 'LanguageID' => $LanguageID});
    return;
}

#========================================================================================
# function     sendTest
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Newsletter->sendTest();
#----------------------------------------------------------------------------------------
# description  sends a test email with the newsletter
#========================================================================================
sub sendTest {
    my $self = shift;

    my $TestEMails = $self->get('NewsletterCampaign')->get('TestEMail');

    my $SampleBillingAddress = {
        'Gender'        => '2',
        'Salutation'    => 'Frau',
        'Title'         => 'Dr.',
        'FirstName'     => 'Maxima',
        'LastName'      => 'Mustermann',
        'Zipcode'       => '12345',
        'City'          => 'Musterstadt',
        'Street'        => 'Musterstrae 2b',
    };

    my $Connection = _getSMTPConnection();
    $Connection->connection( sub {
        foreach my $TestEMail (@{SplitEMailAddresses($TestEMails)}) {
            my $SampleCustomerData = {
                'DisplayName' => 'Maxima Mustermann',
                'EMail' => $TestEMail,
                'BillingAddress' => $SampleBillingAddress,
            };

            # build content with Test Data
            my $processedContent = $self->buildContent( $SampleCustomerData );

            # send Mail
            $self->sendMail($Connection, 'Test Newsletter', $TestEMail, $processedContent);
        }
    });
    return;
}

#========================================================================================
# function     sendNewsletter
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Newsletter->sendNewsletter;
# syntax       $Newsletter->sendNewsletter( $Servlet );
#----------------------------------------------------------------------------------------
# description  sends a newsletter to each campaign recipient.
#----------------------------------------------------------------------------------------
# input        $Servlet | (optional) current servlet | DE_EPAGES::WebInterface::Servlet::BaseServlet
#========================================================================================
sub sendNewsletter {
    my $self = shift;
    my ( $Servlet ) = @_;

    # set in progress flag
    $self->set({'DeliveryInProgress' => 1});

    my $aAddressees = $self->get('NewsletterCampaign')->get('SubscribedAddressees');

    my $Monitor = defined $Servlet ? DE_EPAGES::Presentation::API::Object::ProgressMonitor->new(
        'Servlet'      => $Servlet,
        'PageTypeName' => 'NewsletterMonitor',
        'Object'       => $self,
        'Total'        => scalar @$aAddressees,
        'Current'      => 0,
    ) : undef;

    my $System = LoadRootObject();

    # List of subscribers, which already recieved this newsletter. See $self->addRecipient
    my %Recipients = map {$_->get('EMail') => undef}
        grep {$_->get('WasSent')}
        @{$self->get('Recipients')};
    my $MailsPerRun = MAILS_PER_RUN;

    my $Abort = 0;
    my $Complete = 0;
    my $GotInvalidSender = 0;
    while( !$Complete && !$Abort ) {
        my $MailsPerPackage = $System->get('NewsletterMailsPerPackage') // $System->get('MailsPerPackage');
        my $count = 0;
        my $Connection = _getSMTPConnection();
        $Connection->connection( sub {

            while( my $addressee = shift @$aAddressees ) {
                $Monitor->incCurrent if defined $Monitor;
                my $noHTML = 0;
                my $customerData = {};
                my $User = $addressee->get('User');

                if (defined $User) {
                    $customerData = $User->tleHash;
                    $customerData->{'DisplayName'} = $User->get('Name');
                    $customerData->{'DisplayName'} = $addressee->get('Name') unless defined $customerData->{'DisplayName'};
                    $customerData->{'EMail'}       = $User->get('EMail');
                    $noHTML = 1 unless $User->get('IsHtmlEMailAllowed');
                } else {
                    $customerData->{'DisplayName'} = $addressee->get('Name');
                    $customerData->{'EMail'} = $addressee->get('EMail');
                }

                # don't send email multiple times to the same recipient
                next if exists $Recipients{$customerData->{'EMail'}};

                TriggerHook('OBJ_BeforeSendNewsletter', {
                    'Object' => $self,
                    'NewsletterID' => $self->id,
                    'CustomerData' => $customerData
                });

                my $processedContent = $self->buildContent( $customerData );
                my ($name, $email) = ($customerData->{'DisplayName'}, $customerData->{'EMail'});
                next unless defined $email;

                eval {
                    $self->sendMail($Connection, $name, $email, $processedContent, $noHTML);
                };
                if (ExistsError()) {
                    my $Error = GetError();
                    $Error->addVars({
                        'From'                  => $self->get('SenderEMail'),
                        'To'                    => $email,
                        'Shop'                  => $self->getSite()->alias,
                        'NewsletterCampaign'    => $self->get('NewsletterCampaign')->alias,
                        'Newsletter'            => $self->alias,
                    });
                    LogError($Error);
                    Warning($Error);

                    # Invalid_Sender is fatal the whole campaign is broken.
                    if($Error->code eq 'Invalid_Sender') {
                        $self->set({
                            'Status'             => $NEWSLETTER_ERROR{'INVALID_SENDER'},
                            'DeliveryInProgress' => 0
                        });
                        $Abort = 1;
                        $GotInvalidSender = 1;
                    }
                    # Start a new package, because we don't know the reason for this error.
                    last;
                }

                $self->addRecipient({
                    'WasSent' => 1,
                    'Name'  => $name,
                    'EMail' => $email,
                    'SentDate' => GetCurrentDateTime(),
                });

                if( 0 == $MailsPerRun-- ) { # not complete
                    $Abort = 1;
                    return;
                }

                $Monitor->write if defined $Monitor;

                last if ++$count >= $MailsPerPackage;
            }

            $Complete = 1 if 0 == scalar @$aAddressees;
        });

        my $DelayBetweenPackages = $System->get('DelayBetweenNewsletterPackages') // $System->get('DelayBetweenPackages');

        sleep( $DelayBetweenPackages ) unless $Complete;
    }

    if( $Complete and not $GotInvalidSender ) {
        $self->set({
            'SentDate'           => GetCurrentDateTime(),
            'Status'             => NEWSLETTER_SENT,
            'DeliveryInProgress' => 0
        });
    }

    return;
}

#========================================================================================
# function     buildContent
# state        private
#----------------------------------------------------------------------------------------
# syntax       $processedContent = $Newsletter->buildContent( $hCustomerData );
#----------------------------------------------------------------------------------------
# description  processes content of newsletter for a particular customer
#----------------------------------------------------------------------------------------
# input        $hCustomerData | TLE-Hash with CustomerData | ref.hash
# return       $processedContent | content with replaced TLEs | string
#========================================================================================
sub buildContent {
    my $self = shift;
    my ($hCustomerData) = @_;

    my $Content = $self->get('Content');
    $Content = '' unless defined $Content;
    my $IsHTML = $self->get('IsHTML');
    my $hasHTMLTag = $Content=~m/<html/;
    my $hasBODYTag = $Content=~m/<body/;
    if($IsHTML && ($hasHTMLTag != 1 || $hasBODYTag != 1)){
        $Content = <<HTML;
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
  <html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">
    <head>
      <style type="text/css">
        body {
          font-family: Arial, Verdana, Helvetica;
          font-size: 11px;
          line-height: 140%;
        }
      </style>
    </head>
    <body>
      $Content
    </body>
  </html>
HTML
    }
    return ReplaceTLEs($Content, $hCustomerData);
}

#========================================================================================
# function     sendMail
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Newsletter->sendMail( $Connection, $TOName, $TOMail, $processedContent, $noHTML );
#----------------------------------------------------------------------------------------
# description  actually sends the mail to "TOName <TOMail>" with the given (already
#               processed) content
#----------------------------------------------------------------------------------------
# input        $Connection | SMTP Connection | DE_EPAGES::MailType::API::Object::SmtpConnection
# input        $TOName | Name of Recipient | string
# input        $TOMail | eMail Address of Recipient | string
# input        $ProcessedContent | content with replaced TLEs | string
# input        $noHTML | only send plain text for this customer (optional), default=false | boolean
#========================================================================================
sub sendMail {
    my $self = shift;
    my ($Connection, $TOName, $TOMail, $ProcessedContent, $noHTML) = @_;

    GetLog()->debug("send newsletter to $TOName <$TOMail>");

    my $FromAddress = Email::Address->new($self->get('SenderName'), $self->get('SenderEMail'))->format;
    TestEMailAddress('RecipientEMailAddress', $TOMail);

    my $MailObject = DE_EPAGES::Mail::API::Mail->new({
        'HEADER_ENC'   => 'utf-8',
        'SUBJECT'      => $self->get('Subject'),
        'FROM'         => $FromAddress,
        'NOTIFY'       => 0,
        'REPLY'        => $FromAddress,
        'TO'           => [Email::Address->new($TOName, $TOMail)->format]
    });

    my $aAttachments = $self->folder('Attachments')->children;
    my $Shop         = $self->get('NewsletterCampaign')->getSite();
    my $SitePath     = $Shop->get('PublicPath');
    my $BaseURL      = $Shop->get('WebUrl');
    $ProcessedContent = ReplaceURIWithURL($ProcessedContent, $BaseURL);

    if ($self->get('IsHTML')) {
        if ($noHTML) {
            $MailObject->addText({
                'content'         => $MailObject->convertHtml2Plain($ProcessedContent),
                'encoding'        => 'quoted-printable',
                'type'            => 'text/html',
                'type.charset'    => 'utf-8',
                'html2plain'      => 1,
                'alttext'         => '',
                'altencoding'     => 'quoted-printable',
                'alt.type.charset'=> 'utf-8',
            });
        } else {
            $MailObject->addHTML({
                'content'         => $ProcessedContent,
                'encoding'        => 'quoted-printable',
                'type'            => 'text/html',
                'type.charset'    => 'utf-8',
                'html2plain'      => 1,
                'alttext'         => '',
                'altencoding'     => 'quoted-printable',
                'alt.type.charset'=> 'utf-8',
            });
        }

        foreach my $attachment (@$aAttachments) {

            # compatibility for existing newsletters...
            # do we have a direct attachment file? (File == Alias, and not from products or categories)
            # file is stored in PublicPath of the attachment object
            my $AttachmentPath = $SitePath;
            if ($attachment->get('File') eq $attachment->get('Alias') ) {
                $AttachmentPath = $attachment->get('PublicPath');
            }

            next unless ExistsFile( $AttachmentPath . '/' . $attachment->get('File') );

            if ($attachment->get('IsHTMLAttachment') and not $noHTML) {
                $MailObject->addHTMLAttachment([{
                    'filename'   => $AttachmentPath . '/' . $attachment->get('File'),
                    'includeid'  => $attachment->alias,
                    'type'       => $attachment->get('Type'),
                },]);
            } else {
                $MailObject->addAttachment([{
                    'filename'   => $AttachmentPath . '/' . $attachment->get('File'),
                    'type'       => $attachment->get('Type'),
                },]);
            }
        }
    } else {
        $MailObject->addText({
            'content'     => $ProcessedContent,
            'encoding'    => 'quoted-printable',
            'type'        => 'text/plain',
            'type.charset'=> 'utf-8',
        });

        foreach my $attachment (@$aAttachments) {

            # compatibility for existing newsletters...
            # do we have a direct attachment file? (File == Alias, and not from products or categories)
            # file is stored in PublicPath of the attachment object
            my $AttachmentPath = $SitePath;
            if ($attachment->get('File') eq $attachment->get('Alias') ) {
                $AttachmentPath = $attachment->get('PublicPath');
            }

            next unless ExistsFile( $AttachmentPath . '/' . $attachment->get('File') );

            if (not $attachment->get('IsHTMLAttachment')) {
                $MailObject->addAttachment([{
                    'filename'   => $AttachmentPath . '/' . $attachment->get('File'),
                    'type'       => $attachment->get('Type'),
                },]);
            }
        }
    }

    eval {
        $Connection->send( $MailObject->createMail, $MailObject->getHeader );
    };
    if (ExistsError()) {
        my $Error = GetError();
        $Error->addVars({
            'From'                  => $self->get('SenderEMail'),
            'To'                    => $TOMail,
            'Shop'                  => $self->getSite()->alias,
            'NewsletterCampaign'    => $self->get('NewsletterCampaign')->alias,
            'Newsletter'            => $self->alias,
        });
        Error($Error);
    }
    return;
}

#========================================================================================
# function     _getSMTPConnection
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $Connection = _getSMTPConnection();
#----------------------------------------------------------------------------------------
# description  Returns a new SMTP connection object based on the given pre-defined
#               parameters (either corresponding to a newsletter-related SMTP
#               configuration or mail-related SMTP configuration). (cf. EPG-18009)
#----------------------------------------------------------------------------------------
# return       $Connection | a SMTP object | DE_EPAGES::Mail::API::SMTP
#========================================================================================
sub _getSMTPConnection {
    my $System = LoadRootObject();

    my @aMailAttributes = qw( SMTPServer SMTPPort SMTPLogin SMTPPassword );

    my $hParameters;
    if ( defined $System->get('NewsletterSMTPServer') ) {
        $hParameters->{$_} = $System->get('Newsletter' . $_) foreach (@aMailAttributes);
    } else {
        $hParameters->{$_} = $System->get($_) foreach (@aMailAttributes);
    }

    return DE_EPAGES::MailType::API::Object::SmtpConnection->new( $hParameters );
}

#========================================================================================
# function     addRecipient
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $Recipient = $Newsletter->addRecipient($hInfo);
#----------------------------------------------------------------------------------------
# description  This function adds an recipient to the newsletter.
#----------------------------------------------------------------------------------------
# input        $hInfo | attributes of new recipient
#                       <ul>
#                         <li>Name - name of addressee - varchar(255)</li>
#                         <li>EMail - email address - varchar(255)</li>
#                         <li>WasSent - newsletter was sent (optional) - integer</li>
#                         <li>SentDate - date when newsletter was sent (optional) - DateTime</li>
#                         <li>Result - sent result (optional) - integer</li>
#                       </ul> | ref.hash
# return       $Addressee | recipient object | DE_EPAGES::Newsletter::API::Object::NewsletterRecipient
#========================================================================================
sub addRecipient {
    my $self = shift;
    my ($hInfo) = @_;

    TestHash('RecipientAttributes', $hInfo, {
                'WasSent' =>'Integer',
                'Result'  =>'Integer',
            });

    TestEMailAddress('RecipientEMailAddress', $hInfo->{'EMail'});
    TestObject('RecipientSentDate', $hInfo->{'SentDate'}, 'DateTime') if defined $hInfo->{'SentDate'};

    my $Class = LoadClassByAlias('NewsletterRecipient');
    return $Class->insertObject({
        'Newsletter' => $self,
        'Name'     => $hInfo->{'Name'} ? $hInfo->{'Name'} : $hInfo->{'EMail'},
        'EMail'    => $hInfo->{'EMail'},
        'WasSent'  => $hInfo->{'WasSent'} ? 1 : 0,
        'SentDate' => $hInfo->{'SentDate'},
        'Result'   => $hInfo->{'Result'} // 0,
    });
}

#========================================================================================
# function     getRecipient
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $Recipient = $Newsletter->getRecipient($EMail);
#----------------------------------------------------------------------------------------
# description  Returns a newsletter recipient by email address.
#               Returns undef if there is no recipient with the given email address.
#               If there are multiple recipients with the same email address, returns the
#               first in the list. There is no unique index on the email column for
#               backward compatibility.
#----------------------------------------------------------------------------------------
# input        $EMail | e-mail address | string
# return       $Recipient | newsletter recipient object | DE_EPAGES::Newsletter::API::Object::NewsletterRecipient
#========================================================================================
sub getRecipient {
    my $self = shift;
    my ($EMail) = @_;

    my $Class = LoadClassByAlias('NewsletterRecipient');
    my $aRecipients = $Class->getObjects('ByNewsletterIDEMail', {
        NewsletterID => $self->id,
        EMail => $EMail,
    });

    return $aRecipients->[0];
}

#========================================================================================
# function     default
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = DE_EPAGES::Newsletter::API::Object::Newsletter->default( $Parent, $hInfo );
# example      $hDefaults = $Class->defaultAttributes($Parent, $hInfo); # for insert a new object
#               $hDefaults = $Object->class->defaultAttributes($Object->parent, $hValues); # for an existing object
#----------------------------------------------------------------------------------------
# description  Returns default attribute values for new objects.
#               Used by <function DE_EPAGES::Object::Object::Class::defaultAttributes>
#----------------------------------------------------------------------------------------
# input        $Parent | parent object of the new object | object
# input        $hInfo | attribute name => value hash (passed to insert()) | ref.hash
# return       $hValues | hash with default attribute values (name => value) | ref.hash
#========================================================================================
sub default {
    my $class = shift;
    my ($Parent, $hInfo) = @_;
    my $Campaign = $Parent->parent();
    return {
        'IsHTML'        => 1,
        'SenderName'    => $Campaign->get('SenderName'),
        'SenderEMail'   => $Campaign->get('SenderEMail'),
        'Subject'       => $Campaign->get('Subject'),
        'Status'        => 0,
        'NewsletterCampaign' => $Campaign,
        'DeliveryInProgress' => 0,
    };
}

#========================================================================================
# function     exportXMLElements
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object->exportXMLElements($Driver, $hAllAttributes);
#----------------------------------------------------------------------------------------
# description  Exports recipients of the newsletter to the xml SAX driver.
#----------------------------------------------------------------------------------------
# input        $Driver | export driver | object
# input        $hAllAttributes | all attributes of object,
#                                 see $Object-><function allAttributes> | ref.hash
#========================================================================================
sub exportXMLElements {
    my $self = shift;
    my ($Driver, $hAttributes) = @_;

    # export language tags
    $self->SUPER::exportXMLElements($Driver, $hAttributes);

    # do only export child objects if ExportRecursive
    if($Driver->{'ExportRecursive'} && $Driver->exportLevel == 1) {

        $Driver->addObject($_->tagNameXML, $_) foreach sort {
            $a->get('EMail') cmp $b->get('EMail')
        } @{$self->get('Recipients')};
    }

    return;
}

sub importXML {
    my $class = shift;
    my ($Handler, $hElement, $Class, $hAttributes) = @_;

    my $self = $class->SUPER::importXML($Handler, $hElement, $Class, $hAttributes);

    $Handler->pushID('Newsletter', $self);

    return $self;
}

sub importXMLEnd {
    my $self = shift;
    my ($Handler, $hElement) = @_;

    $Handler->popID('Newsletter');

    $self->SUPER::importXMLEnd($Handler, $hElement);
    return;
}

sub deleteXML {
    my $class = shift;
    my ($Handler, $hElement, $Class, $hAttributes) = @_;

    my $self = $class->SUPER::deleteXML($Handler, $hElement, $Class, $hAttributes);

    #push object related data only if not deleted
    $Handler->pushID('Newsletter', $self) if defined $self;

    return $self;
}

sub deleteXMLEnd {
    my $self = shift;
    my ($Handler, $hElement) = @_;

    $Handler->popID('Newsletter');

    $self->SUPER::deleteXMLEnd($Handler, $hElement);
    return;
}

1;
