#==============================================================================
# package      DE_EPAGES::Newsletter::UI::Newsletter
# state        private
#==============================================================================
package DE_EPAGES::Newsletter::UI::Newsletter;
use base qw( DE_EPAGES::Presentation::UI::Object );

use strict;

use DE_EPAGES::Core::API::PerlTools qw (GetLocalTimeZone ElementOf);
use DE_EPAGES::Object::API::Factory qw (LoadRootObject LoadClassByAlias);
use DE_EPAGES::Core::API::File qw (CopyFile);
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( LogError );
use DE_EPAGES::Newsletter::API::Constants qw (
    NEWSLETTER_SCHEDULED
    NEWSLETTER_EDITABLE
    %NEWSLETTER_ERROR );
use DE_EPAGES::TLE::API::TLE qw( TestSyntax );

#==============================================================================
# Event handlers
#==============================================================================
sub Save {
    my $self = shift;
    my ($Servlet) = @_;


    $Servlet->form->form($Servlet->object(), 'Newsletter');
    $self->SUPER::Save($Servlet);

    my ($ScheduleDate) = $Servlet->form->value('ScheduleDate');
    my $Newsletter = $Servlet->object;

    unless ($self->_isStatusError()){
        if ($ScheduleDate) {
            $Newsletter->set({'Status' => NEWSLETTER_SCHEDULED});
        } else {
            $Newsletter->set({'Status' => NEWSLETTER_EDITABLE});
        }
    }
}

sub _isStatusError {
    my $self = shift;
    my $Newsletter = $self->object;

    return ElementOf($Newsletter->get('Status'), values %NEWSLETTER_ERROR);
}

sub NewNewsletter {
    my $self    = shift;
    my ($Servlet) = @_;
    my $NewsletterCampaign    = $Servlet->object;

    my $Site = $NewsletterCampaign->getSite();
    my $LanguageID = $NewsletterCampaign->get('LangID') || $Site->get('LanguageID');
    if (not defined $LanguageID) {
        $LanguageID = $Servlet->languageID;
        $Site->set({'LanguageID'=> $LanguageID});
    }

    my $Folder  = $NewsletterCampaign->folder('Newsletters');
    my $Class   = LoadClassByAlias ('Newsletter');
    return $self->SUPER::NewObject($Servlet, $Folder, $Class);
}

sub CloneNewsletter {
    my $self    = shift;
    my ($Servlet) = @_;

    my $NewsletterCampaign = $Servlet->object;
    my $Form = $Servlet->form;

    my $Site = $NewsletterCampaign->getSite();
    my $hValues = $Form->form($NewsletterCampaign, 'ListedObjects');

    my @NewsletterIDs = map { $_->{'ListObjectID'} } @{$hValues->{'ListObjectIDs'}};
    my $aNewsletters = $Site->loadObjects( \@NewsletterIDs, 'Newsletter' );

    foreach my $Newsletter (@$aNewsletters) {
        my $NewAlias = $Newsletter->alias;
        my $Folder = $Newsletter->parent;
        my $cnt = 1;

        # test if Alias has already an ext like [1] and replace ext if exists
        if ($NewAlias =~ /.*\[\d{1,}\]$/gx){
            while ($Folder->existsChild($NewAlias)){
                $cnt++;
                $NewAlias =~ s/(\[)\d{1,}(\])$/$1$cnt$2/gx;
            }
        }else{
            while ($Folder->existsChild($NewAlias."[$cnt]")){
                $cnt++;
            }
            $NewAlias .= "[$cnt]";
        }
        # clone the product and set cloned product invisible
        my $Clone = $Newsletter->clone({
            'Alias'    => $NewAlias,
            'SentDate' => undef,
            'Status'   => 0,
            'ScheduleDate' => undef,
        });
        foreach my $attachment (@{$Newsletter->folder('Attachments')->children}) {
            my $NewAttachment = $Clone->folder('Attachments')->insertChild('NewsletterAttachment', {
                'Alias' => $attachment->alias,
                'IsHTMLAttachment' => $attachment->get('IsHTMLAttachment'),
                'File' => $attachment->get('File'),
            });

            # backward compatibility for existing newsletters
            # do we have a direct attachment file? (File == Alias, and not from products or categories)
            # file is stored in PublicPath of the attachment object and is cloned to the PublicPath of the clone
            if ($attachment->get('File') eq $attachment->get('Alias')){
                # duplicate direct attachment files
                my $SourceFile  = $attachment->get('PublicPath') . '/' . $attachment->get('File');
                my $TargetFile  = $NewAttachment->get('PublicPath') . '/' . $attachment->get('File');
                CopyFile( $SourceFile, $TargetFile, {'create_directory' => 1} );
            }

        }
    }

    return;
}

sub SendTest {
    my $self    = shift;
    my ($Servlet) = @_;

    $self->Save($Servlet);
    my $Newsletter = $Servlet->object;

    eval {
        $Newsletter->sendTest();
    };
    if (ExistsError()) {
        my $Error = GetError();
        LogError($Error);
        $self->_handleTestError($Error);
    } else {
        $self->_updateStatus($Newsletter);
    }

    return;
}

sub _updateStatus {
    my $self = shift;
    my ($Newsletter) = @_;
    if ($Newsletter->get('ScheduleDate')) {
        $Newsletter->set({'Status' => NEWSLETTER_SCHEDULED});
    } else {
        $Newsletter->set({'Status' => NEWSLETTER_EDITABLE});
    }
}

sub _handleTestError {
    my $self = shift;
    my ($Error) = @_;
    my $Servlet = $self->servlet();
    my $Newsletter = $self->object();

    if ( $Error->code() eq 'Invalid_Sender' ) {
        $Servlet->form->addFormError( {
                'Reason' => 'InvalidSender',
                'Name'   => 'InvalidSender',
        });
        $Newsletter->set( { 'Status' => $NEWSLETTER_ERROR{INVALID_SENDER} } );
    }
    else {
        $Servlet->form->addFormError( {
                'Reason' => 'TestMailError',
                'Name'   => 'TestMailError',
        });
        $Newsletter->set( { 'Status' => $NEWSLETTER_ERROR{ERROR} } );
    }
    $Servlet->form->executeFormError;
}

sub SendNewsletter {
    my $self    = shift;
    my ($Servlet) = @_;

    # save first
    $self->Save($Servlet);

    # confirmation
    if (not $Servlet->params('SEND_CONFIRMATION')) {
        $Servlet->vars('SEND_CONFIRMATION' => 1);
        return;
    }

    my $Newsletter = $Servlet->object;
    # EPG-18009: allow newsletter specific settings
    my $System = LoadRootObject();
    my $MailsPerPackage = $System->get('NewsletterMailsPerPackage') // $System->get('MailsPerPackage');

    my $aAddressees = $Newsletter->get('NewsletterCampaign')->get('SubscribedAddressees');
    my $IsScheduled = scalar(@$aAddressees) > $MailsPerPackage  ? 1:0;


    if (not $IsScheduled){
        $Newsletter->sendNewsletter( $Servlet );
    } else {
        #schedule now to prevent heavy traffic on the appserver
        my $now = DateTime->now()->set_time_zone(GetLocalTimeZone());
        $Newsletter->set({'ScheduleDate' => $now});
        $Newsletter->set({'Status' => NEWSLETTER_SCHEDULED});
        $Servlet->vars('SEND_SCHEDULED' => 1);
    }

    return;
}

sub attributeValues {
    my $self = shift;
    my ( $Servlet, $Object, $Class ) = @_;

    my $hhValues = $self->SUPER::attributeValues( $Servlet, $Object, $Class );

    # don't test if a new newsletter should created
    return $hhValues unless $Object->instanceOf('Newsletter');

    my $Content = $hhValues->{'Values'}->{'Content'};
    if( defined $Content && $Content =~ /\S/ ) {
        if( TestSyntax( \$Content ) ) {
            $Servlet->form->addFormError({
                'Reason'=> 'TLEError',
                'Name'  => 'Content',
            });
        }
    }

    unless( grep { $_->{'Reason'} =~ /DATETIME/ }  @{$Servlet->form->formErrors} ) {
        my $ScheduleDate = $hhValues->{'Values'}->{'ScheduleDate'};
        if( defined $ScheduleDate && $ScheduleDate < DateTime->now ) {
            $Servlet->form->addFormError({
                'Reason'=> 'DateInPast',
                'Name'  => 'ScheduleDate',
            }) ;
        }
    }

    $Servlet->form->executeFormError;

    return $hhValues;
}

1;
