use strict;
use Getopt::Long;
use Time::HiRes qw (time);
use DE_EPAGES::Core::API::Error qw( GetError ExistsError );
use DE_EPAGES::Core::API::Log qw( LogError );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( RunOnStore );
use DE_EPAGES::Object::API::Factory qw ( LoadObject );
use DE_EPAGES::Order::API::Cartridge;
use DE_EPAGES::Shop::API::System qw( GetUnityShopIDs );
use DE_EPAGES::WebInterface::API::MessageCenter qw( SynchronizeCache );

#========================================================================================
# function         FixNewOrdersNotification
# state            public
#----------------------------------------------------------------------------------------
# syntax           FixNewOrdersNotification();
#----------------------------------------------------------------------------------------
# description      Activates the notification and delete all objects from it
#                   and this for all Unity shops
#========================================================================================
sub FixNewOrdersNotification {

    print "Reading all Unity shops\n";
    my $aShopIDs = GetUnityShopIDs();

    foreach my $ShopID (@$aShopIDs) {
        my $Shop = LoadObject($ShopID);

        # nothing to do?
        next unless $Shop->folder('ShopNotifications')->existsChild('NewOrders');

        # clean and activate
        $Shop->folder('ShopNotifications')->child('NewOrders')->set({'ShopNotificationObjectIDString' => undef,
                                                                     'IsActive' => 1,
                                                                    });
    }

    eval { SynchronizeCache(); };
    LogError(GetError()) if ExistsError();

    return;
}

sub Main {

    local $| = 1;

    my $Password;
    my $StoreName;
    my $Help;

    GetOptions(
        'help'        => \$Help,
        'passwd=s'    => \$Password,
        'storename=s' => \$StoreName,
    );

    usage() if $Help;

    unless ( defined $StoreName ) {
        print STDERR "Missing parameter -storename.\n\n";
        usage();
    }
    my $time1 = time;

    RunOnStore(
        Store      => $StoreName,
        DBPassword => $Password,
        Sub        => sub { FixNewOrdersNotification(); },
    );
    my $time2 = time;
    printf "need %.1f seconds\n", $time2 - $time1;
}

sub usage {
    print <<END_USAGE;
Description:
    Resets the new order notification for all Unity shops
    to 0 objects and activate the notification
    (we need an always active notification and get the count of unseen orders
    by a using a tle function and a search definition
    because of appserver sync and table locking problems (UNITY-6182))

Usage:
    perl $0 [options] [flags] datafiles

options:
    -passwd     database user password                          ()
    -storename  name of store                                   ()
flags:
    -help       show the command line options

Example:
    perl $0 -storename Store
END_USAGE
    exit 2;
}

RunScript( 'Sub' => \&Main );
