#==================================================================================================
# package      DE_EPAGES::Paybox::API::GatewayInterface::Implementations::PaymentGateway
# base         DE_EPAGES::Paybox::API::GatewayInterface::PaymentGateway
# state        public
# description  Test PaymentGateway implementation.
#==================================================================================================
package DE_EPAGES::Paybox::API::GatewayInterface::Implementations::PaymentGateway;
use base DE_EPAGES::Paybox::API::GatewayInterface::PaymentGateway;

use strict;

use Crypt::OpenSSL::RSA;
use Data::Dumper;
use MIME::Base64;
use URI::Escape;
use Digest::SHA qw (
    hmac_sha512_hex
);
use LWP::UserAgent;

use DE_EPAGES::Core::API::Error qw (
    Error
);
use DE_EPAGES::Core::API::Object::DateTimeFormatter;
use DE_EPAGES::Object::API::Factory qw (
    ExistsObjectByGUID
    LoadObjectByGUID
    LoadRootObject
);
use DE_EPAGES::Core::API::PerlTools qw (
    GetCurrentDateTime
    isValid
);
use DE_EPAGES::Shop::API::Url qw (
    BuildShopUrl
);
use DE_EPAGES::Core::API::Log qw (
    GetLog
    LogCommunication
);
use DE_EPAGES::Paybox::API::Constants qw (
    STATUS_ACCEPTED
    STATUS_PAID
    STATUS_CANCELLED
    ABBR_ACCEPTED
    ABBR_REJECTED
    ABBR_CANCELLED
    ABBR_WAITING
    CODE_ACCEPTED
    CODE_CANCELLED
    CODE_WAITING
    $hProductVariantCodes
    $hPaymentResponseOperationMap
);
use feature 'switch';
#==================================================================================================
# function     getPaymentRequestURL
# state        public
#--------------------------------------------------------------------------------------------------
# syntax       my $RequestURL = $PaymentGateway->getPaymentRequestURL();
#--------------------------------------------------------------------------------------------------
# description  Returns a PaymentGateway Test Request URL
#--------------------------------------------------------------------------------------------------
# return       $RedirectURL | The GatewayPayment Test URL | string
#==================================================================================================
sub getPaymentRequestURL {
    my $self = shift;

    GetLog('Paybox')->debug('PayboxPaymentGateway: getPaymentRequestURL()');

    my $Configurator = $self->getConfigurator();
    my $Paybox = $Configurator->getProperty('Paybox');

    return $Paybox->get('IsInTestMode')?
                $Configurator->getProperty('RequestURLTest') :
                $Configurator->getProperty('RequestURL')
}

#==================================================================================================
# function     getPayboxDirectURL
# state        public
#--------------------------------------------------------------------------------------------------
# syntax       my $PayboxDirectURL = $PaymentGateway->getPayboxDirectURL();
#--------------------------------------------------------------------------------------------------
# description  Returns the PayboxDirect URL which can be used to perform PayboxDirect requests.
#--------------------------------------------------------------------------------------------------
# return       $PayboxDirectURL | PayboxDirect URL | string
#==================================================================================================
sub getPayboxDirectURL {
    my $self = shift;

    GetLog('Paybox')->debug('PayboxPaymentGateway: getPayboxDirectURL()');

    my $Configurator = $self->getConfigurator();
    my $Paybox = $Configurator->getProperty('Paybox');

    return $Paybox->get('IsInTestMode')?
                $Configurator->getProperty('PayboxDirectURLTest') :
                $Configurator->getProperty('PayboxDirectURL');
}

#==================================================================================================
# function     getPaymentRequestData
# state        public
#--------------------------------------------------------------------------------------------------
# syntax       $aData = $PaymentGateway->getPaymentRequestData($LineItemPayment);
#--------------------------------------------------------------------------------------------------
# description  Returns a test data for a payment request
#--------------------------------------------------------------------------------------------------
# input        $LineItemPayment | A LineItemPayment object to prepare the request data | object
#--------------------------------------------------------------------------------------------------
# return       $RequestData | Array of N hash items { Field, Value } | array.ref
#==================================================================================================
sub getPaymentRequestData {
    my $self = shift;
    my ($LineItemPayment) = @_;

    GetLog('Paybox')->debug('PayboxPaymentGateway: getPaymentRequestFormData()');

    my $PaymentMethod = $LineItemPayment->get('PaymentMethod');
    my $aData = [];

    if (defined $LineItemPayment) {
        my $Configurator = $self->getConfigurator();
        my $Shop = $LineItemPayment->getSite();
        $aData = [
            {
                'Field' => 'PBX_SITE',
                'Value' => $Configurator->getProperty('SiteNumber')
            }, {
                'Field' => 'PBX_RANG',
                'Value' => $Configurator->getProperty('RankNumber')
            }, {
                'Field' => 'PBX_IDENTIFIANT',
                'Value' => $Configurator->getProperty('Identifier')
            }, {
                'Field' => 'PBX_TOTAL',
                'Value' => $self->_getMinorUnitAmount($Shop, $LineItemPayment),
            }, {
                'Field' => 'PBX_DEVISE',
                'Value' => $self->_getCurrencyCode($Shop, $LineItemPayment)
            }, {
                'Field' => 'PBX_CMD',
                'Value' => $PaymentMethod->id.'_'.$LineItemPayment->get('TransID')
            }, {
                'Field' => 'PBX_TYPEPAIEMENT',
                'Value' => 'CARTE'
            }, {
                'Field' => 'PBX_TYPECARTE',
                'Value' => $Configurator->getProperty('TypeMethod')->get('PaymentTypeKey')
            }, {
                'Field' => 'PBX_AUTOSEULE',
                'Value' => $self->_getProductVariant()
            }, {
                'Field' => 'PBX_PORTEUR',
                'Value' => $Shop->get('Address')->get('EMail')
            }, {
                # PBX_RETOUR contains the names of all parameters which will be send back to ePages
                # in response messages.
                'Field' => 'PBX_RETOUR',
                'Value' => 'Code:E;Amount:M;PaymentId_TransId:R;TransId2:S;Auto:A;CallNumber:T;Pays:Y;Sign:K'
            }, {
                'Field' => 'PBX_HASH',
                'Value' => 'SHA512'
            }, {
                'Field' => 'PBX_TIME',
                'Value' => $self->_getCurrentDateTime()
            }, {
                'Field' => 'PBX_REPONDRE_A',
                'Value' => $self->_buildCallbackURL($Shop)
            }, {
                'Field' => 'PBX_EFFECTUE',
                'Value' => $self->_buildBacklinkURL($Shop, 'A')
            }, {
                'Field' => 'PBX_REFUSE',
                'Value' => $self->_buildBacklinkURL($Shop, 'R')
            }, {
                'Field' => 'PBX_ANNULE',
                'Value' => $self->_buildBacklinkURL($Shop, 'C')
            }, {
                'Field' => 'PBX_ATTENTE',
                'Value' => $self->_buildBacklinkURL($Shop, 'W')
            }
        ];

        my $HMACKey = $Configurator->getProperty('HMACKey');
        my $HMAC = $self->_calculateHMAC($aData, $HMACKey);

        push(@$aData, { 'Field' => 'PBX_HMAC', 'Value' => $HMAC });
    }

    return $aData;
}

#==================================================================================================
# _getCurrencyCode
#==================================================================================================
sub _getCurrencyCode {
    my $self = shift;
    my ($Shop, $LineItemPayment) = @_;

    my $CurrencyID = $LineItemPayment->get('CurrencyID');
    my $Currencies = $Shop->get('Currencies');
    foreach my $Currency (@$Currencies) {
        if ($Currency->{'CurrencyID'} eq $CurrencyID) {
            return $Currency->{'NumCode'};
        }
    }

    return;
}

#==================================================================================================
# _getProductVariant
#==================================================================================================
sub _getProductVariant {
    my $self = shift;

    my $Configurator = $self->getConfigurator();
    my $ProductVariant = $Configurator->getProperty('ProductVariant');

    return $hProductVariantCodes->{$ProductVariant} ;
}

#==================================================================================================
# _getCurrentDateTime
#==================================================================================================
sub _getCurrentDateTime {
    my $self = shift;

    my $Formatter = DE_EPAGES::Core::API::Object::DateTimeFormatter->new();
    return $Formatter->format_datetime( GetCurrentDateTime(), 'date', '%Y-%m-%dT%H:%M:%S' );
}

#==================================================================================================
# _buildCallbackURL
#==================================================================================================
sub _buildCallbackURL {
    my $self = shift;
    my ($Shop) = @_;

    return  $self->_buildResponseURL(
                $Shop,
                { 'ChangeAction'  => 'PbCb' }
            );
}

#==================================================================================================
# _buildBacklinkURL
#==================================================================================================
sub _buildBacklinkURL {
    my $self = shift;
    my ($Shop, $ReturnValue) = @_;

    return  $self->_buildResponseURL(
                $Shop,
                {
                    'ChangeAction'  => 'PbBl',
                    'Ret'           => $ReturnValue
                }
            );
}

#==================================================================================================
# _buildCallbackURL
#==================================================================================================
sub _buildResponseURL {
    my $self = shift;
    my ($Shop, $hParameters) = @_;

    my $System = $self->_loadRootObject();
    my $UseSSL = $System->get('Paybox_UseSSL');

    return
        BuildShopUrl(
            $Shop,
            $hParameters,
            {
                'UseSSL' => $UseSSL,
                'UseObjectPath'=> 0
            }
        );
}

#==================================================================================================
# function     _calculateHMAC
# state        private
#--------------------------------------------------------------------------------------------------
# syntax       $HMAC = $PaymentGateway->_calculateHMAC($aData, $SecretKey);
#--------------------------------------------------------------------------------------------------
# description  Calculates the HMAC (Hash Message Authentification Code) to include in the
#               Paybox request. The HMAC is calculated with the ordered PBX fields in the request
#               form and a SecretKey (see Paybox integration manual)
#--------------------------------------------------------------------------------------------------
# input        $aData | Array with the ordered PBX fields for the request form | array.ref
# input        $SecretKey | The Paybox SecretKey | string
#--------------------------------------------------------------------------------------------------
# return       $HMAC | The HMAC calculated for the aData | string
#==================================================================================================
sub _calculateHMAC {
    my $self = shift;
    my ($aData, $SecretKey) = @_;

    my $Message;
    foreach my $item ( @$aData) {
        if (defined $Message) {
            $Message .= '&';
        }
        $Message .= $item->{'Field'}.'='.$item->{'Value'};
    }

    my $BinKey = pack('H*', $SecretKey);
    my $HMAC= uc(hmac_sha512_hex($Message, $BinKey));

    return $HMAC;
}

#==================================================================================================
# function     processPaymentResponse
# state        public
#--------------------------------------------------------------------------------------------------
# syntax       $PaymentGateway->processPaymentResponse($hPaymentContext);
#--------------------------------------------------------------------------------------------------
# description  Validates the callback request (direct Paybox call) by checking the query string
#               and its SHA1 signature. If it is OK then the payment process is delegated to  a
#               PaymentDispatcher object
#--------------------------------------------------------------------------------------------------
# input        $hPaymentContext | The information needed to process the request:
#                       Shop            The Shop object for the request
#                       Query           The request paramas (Servlet->paramas)
#                       Dispatcher      The PaymentDispatcher object to delegate the process
#                   | hash.ref
#==================================================================================================
sub processPaymentResponse {
    my $self = shift;
    my ($hPaymentContext) = @_;

    my $Shop = $hPaymentContext->{'Shop'};
    my $CallbackParams = $hPaymentContext->{'CallbackParams'};

    if ( $self->_checkCallbackSignature($CallbackParams->getQueryString()) ) {
        $hPaymentContext->{'Dispatcher'}->processPayment({
            'Shop'          => $Shop,
            'Operation'     => $self->_mapPayboxCodeToOperation($CallbackParams->getCode()),
            'PayboxCode'    => $CallbackParams->getCode(),
            'Amount'        => $CallbackParams->getAmount(),
            'TransID'       => $CallbackParams->getTransID(),
            'TransID2'      => $CallbackParams->getTransID2(),
            'Authorization' => $CallbackParams->getAuto(),
            'CallNumber'    => $CallbackParams->getCallNumber(),
            'Pays'          => $CallbackParams->getPays(),
        });
    } else {
        LogCommunication('processPaymentResponse: Wrong Payment Request Signature', $CallbackParams, 'Paybox');
    }

    return;
}

#==================================================================================================
# function     _mapPayboxCodeToOperation
# state        public
#--------------------------------------------------------------------------------------------------
# syntax       $Operation = $PaymentGateway->_mapPayboxCodeToOperation($PayboxCode);
#--------------------------------------------------------------------------------------------------
# description  Maps the Paybox code to the Payment related operation (ACCEPTED, CANCELLED, ...)
#--------------------------------------------------------------------------------------------------
# input        $PayboxCode | The Paybox result code | string
#--------------------------------------------------------------------------------------------------
# return       $Operation | The Payment operation code | string
#==================================================================================================
sub _mapPayboxCodeToOperation {
    my $self = shift;
    my ($PayboxCode) = @_;

    my $Operation = ABBR_REJECTED;
    if ($PayboxCode eq CODE_ACCEPTED) {
        $Operation = ABBR_ACCEPTED;
    } elsif ($PayboxCode eq CODE_CANCELLED) {
        $Operation = ABBR_CANCELLED;
    } elsif ($PayboxCode eq CODE_WAITING) {
        $Operation = ABBR_WAITING;
    }

    return $hPaymentResponseOperationMap->{$Operation};
}

#==================================================================================================
# function     checkPaymentResponse
# state        public
#--------------------------------------------------------------------------------------------------
# syntax       $PaymentGateway->checkPaymentResponse($hPaymentContext);
#--------------------------------------------------------------------------------------------------
# description  Validates the backlink request (browser redirect) by checking the query string
#               and its SHA1 signature. A hash contained the corresponding order or error is
#               returned.
#--------------------------------------------------------------------------------------------------
# input        $hPaymentContext | The information needed to process the request:
#                       Shop            The Shop object for the request
#                       Query           The request paramas (Servlet->paramas)
#                       Dispatcher      The PaymentDispatcher object to delegate the process
#                   | hash.ref
#--------------------------------------------------------------------------------------------------
# return       $hResult | The result of the operation. | hash.ref
#==================================================================================================
sub checkPaymentResponse {
    my $self = shift;
    my ($hPaymentContext) = @_;
    my $Shop                = $hPaymentContext->{'Shop'};
    my $CallbackParams      = $hPaymentContext->{'CallbackParams'};
    my $IsSignatureValid    = $self->_checkBacklinkSignature($CallbackParams->getQueryString());
    my $Success             = $CallbackParams->operationSuccessful();
    my $LineItemPaymentGUID = $CallbackParams->getTransID();
    my $ObjectExists        = $self->_existsObjectByGUID($LineItemPaymentGUID);
    my $Order;
    my $ErrorCode;
    my $WebUrlUnitySSL;

    if ( not $Success ) {
        GetLog('Paybox')->error('checkPaymentResponse: Payment not successful');
        $ErrorCode = 'PaymentProcess_Canceled';
    }
    elsif ( not $IsSignatureValid ) {
        GetLog('Paybox')->error('checkPaymentResponse: Wrong Payment Request Signature');
        $ErrorCode = 'CreatePayment_Failed'
    }
    elsif ( not $ObjectExists ) {
        GetLog('Paybox')->error('checkPaymentResponse: Invalid LineItemPayment GUID');
        $ErrorCode = 'CreatePayment_Failed';
    }
    else {
        my $LineItemPayment = $self->_loadObjectByGUID($LineItemPaymentGUID);
        my $TransStatus = $LineItemPayment->get('TransStatus');

        if ( not isValid($TransStatus) ) {
            GetLog('Paybox')->error('checkPaymentResponse: Transaction status was not set. ' .
                                    'Looks like no callback was done by Paybox.');
            $ErrorCode = 'PaymentProcess_StatusUnknown';
        }
        elsif ($TransStatus ne STATUS_ACCEPTED and $TransStatus ne STATUS_PAID) {
            GetLog('Paybox')->error('checkPaymentResponse: Invalid transaction status '.$TransStatus);
            $ErrorCode = 'PaymentProcess_Failed';
        }
    }

    if ( defined $ObjectExists ) {
        my $LineItemPayment = $self->_loadObjectByGUID($LineItemPaymentGUID);
        $Order = $LineItemPayment->parent()->parent();
        $WebUrlUnitySSL = $self->_getWebUrlUnitySSL($LineItemPayment);
        $self->_addPaymentErrorCode($LineItemPayment, $ErrorCode);
    }

    return {
        'Order'             => $Order,
        'ErrorCode'         => $ErrorCode,
        'ErrorViewObject'   => $Shop,
        'WebUrlUnitySSL'    => $WebUrlUnitySSL
    };
}

#==================================================================================================
# _checkBacklinkSignature
#==================================================================================================
sub _checkBacklinkSignature {
    my $self = shift;
    my ($QueryString) = @_;

    return  $self->_checkSignature({
                'Message'   => ($QueryString =~ m/^(.*)&Sign=/),
                'Signature' => ($QueryString =~ m/&Sign=(.*)/)
            });
}

#==================================================================================================
# _checkCallbackSignature
#==================================================================================================
sub _checkCallbackSignature {
    my $self = shift;
    my ($QueryString) = @_;

    return  $self->_checkSignature({
                'Message'   => ($QueryString =~ m/&(Code=.*)&Sign=/),
                'Signature' => ($QueryString =~ m/&Sign=(.*)/)
            });
}

#==================================================================================================
# _checkSignature
#==================================================================================================
sub _checkSignature {
    my $self = shift;
    my ($hData) = @_;

    my $PayboxPublicKey = $self->getConfigurator()->getProperty('PublicKey');
    my $RSA_pub = Crypt::OpenSSL::RSA->new_public_key($PayboxPublicKey);
    my $Signature = uri_unescape($hData->{'Signature'});
    return  $RSA_pub->verify(
                $hData->{'Message'},
                decode_base64($Signature)
            );
}

#==================================================================================================
# function     capturePayment
# state        public
#--------------------------------------------------------------------------------------------------
# syntax       $PaymentGateway->capturePayment($Shop, $LineItemPayment);
#--------------------------------------------------------------------------------------------------
# description  Captures a payment that has previously been authorized.
#--------------------------------------------------------------------------------------------------
# input        $Shop | shop in which the payment has been authorized | object
# input        $LineItemPayment | line item that represents the authorized payment | object
#--------------------------------------------------------------------------------------------------
# return       $hResult | The result of the operation.
#                       success         1 if the operation was successful
#                       error_message   error message in case an error occured
#                   | hash.ref
#==================================================================================================
sub capturePayment {
    my $self = shift;
    my ($Shop, $LineItemPayment) = @_;

    my $BasketOrder = $LineItemPayment->parent->parent;
    my $hRequestParams = $self->_getPayboxDirectRequestParams($Shop, $LineItemPayment);
    $hRequestParams->{'TYPE'} = '00002'; # Request type for capture requests

    LogCommunication('Capture request parameters', $hRequestParams, 'Paybox');

    my $UserAgent = $self->_createUserAgent();
    my $Response = $UserAgent->post($self->getPayboxDirectURL(), $hRequestParams);
    my $hResult = $self->_processPayboxDirectResponse($Response);

    LogCommunication('Capture result', $hResult, 'Paybox');

    if ($hResult->{'success'}) {
        $BasketOrder->set({'PaidOn' => DateTime->now});
        $LineItemPayment->set({
            'TransStatus' => STATUS_PAID,
            'IsCaptured'  => 1,
        });
    }

    return $hResult;
}

#==================================================================================================
# function     cancelPayment
# state        public
#--------------------------------------------------------------------------------------------------
# syntax       $PaymentGateway->cancelPayment($Shop, $LineItemPayment);
#--------------------------------------------------------------------------------------------------
# description  Cancels a payment that has previously been captured.
#--------------------------------------------------------------------------------------------------
# input        $Shop | shop in which the payment has been authorized | object
# input        $LineItemPayment | line item that represents the authorized payment | object
#--------------------------------------------------------------------------------------------------
# return       $hResult | The result of the operation.
#                       success         1 if the operation was successful
#                       error_message   error message in case an error occured
#                   | hash.ref
#==================================================================================================
sub cancelPayment {
    my $self = shift;
    my ($Shop, $LineItemPayment) = @_;

    my $BasketOrder = $LineItemPayment->parent->parent;
    my $hRequestParams = $self->_getPayboxDirectRequestParams($Shop, $LineItemPayment);
    $hRequestParams->{'TYPE'} = '00005'; # Request type for cancel requests

    LogCommunication('Cancel request parameters', $hRequestParams, 'Paybox');

    my $UserAgent = $self->_createUserAgent();
    my $Response = $UserAgent->post($self->getPayboxDirectURL(), $hRequestParams);
    my $hResult = $self->_processPayboxDirectResponse($Response);

    LogCommunication('Cancel result', $hResult, 'Paybox');

    if ($hResult->{'success'}) {
        $BasketOrder->set({
            'CancelledOn' => DateTime->now,
            'PaidOn'      => undef,
        });
        $LineItemPayment->set({
            'TransStatus' => STATUS_CANCELLED,
            'IsCancelled' => 1,
        });
    }

    return $hResult;
}

#==================================================================================================
# _getPayboxDirectRequestParams
#==================================================================================================
sub _getPayboxDirectRequestParams {
    my $self = shift;
    my ($Shop, $LineItemPayment) = @_;

    my $hLineItemPaymentInfo = $LineItemPayment->get([
        'Amount',
        'CurrencyID',
        'PaymentMethod',
        'TransID',
        'TransID2',
        'PayboxCallNumber',
        'Field1',
        'Field2',
    ]);

    my $PaymentMethod = $hLineItemPaymentInfo->{'PaymentMethod'};
    my $hPaymentMethodInfo = $PaymentMethod->get([
        'PayboxSiteNumber',
        'PayboxRankNumber',
        'PayboxHMACKey',
        'PayboxMerchantPassword',
    ]);

    my $hRequestParams = {
        'VERSION' => '00104',
        'SITE' => $hPaymentMethodInfo->{'PayboxSiteNumber'},
        'RANG' => $hPaymentMethodInfo->{'PayboxRankNumber'},
        'CLE' => $hPaymentMethodInfo->{'PayboxMerchantPassword'},
        'NUMQUESTION' => $self->_getNumquestion(),
        'MONTANT' => $self->_getMinorUnitAmount($Shop, $LineItemPayment),
        'DEVISE' => $self->_getCurrencyCode($Shop, $LineItemPayment),
        'REFERENCE' => $hLineItemPaymentInfo->{'TransID'},
        'NUMTRANS' => $hLineItemPaymentInfo->{'TransID2'},
        'NUMAPPEL' => $hLineItemPaymentInfo->{'PayboxCallNumber'},
        'DATEQ' => $self->_getFormattedDate(),
        'PAYS' => $hLineItemPaymentInfo->{'Field1'},
    };

    return $hRequestParams;
}

#==================================================================================================
# _processPayboxDirectResponse
#==================================================================================================
sub _processPayboxDirectResponse {
    my $self = shift;
    my ($Response) = @_;

    my $hResult = {};

    if ($Response->is_success) {
        my $hParsedResponse = $self->_parseResponse($Response);
        GetLog('Paybox')->debug('Response: '.Dumper($hParsedResponse));

        # CODEREPONSE of 00000 means no error occured
        if ($hParsedResponse->{'CODEREPONSE'} eq '00000') {
            $hResult->{'success'} = 1;
        }
        else {
            $hResult->{'success'} = 0;
            $hResult->{'error_message'} = $hParsedResponse->{'COMMENTAIRE'};
        }
    }
    else {
        $hResult->{'success'} = 0;
        $hResult->{'error_message'} = $Response->message;
    }

    return $hResult;
}

#==================================================================================================
# _createUserAgent
#==================================================================================================
sub _createUserAgent {
    my $self = shift;

    my $ua = LWP::UserAgent->new();
    $ua->timeout(60); ## no critic qw(ProhibitMagicNumbers);

    return $ua;
}

#==================================================================================================
# _getFormattedDate
#==================================================================================================
sub _getFormattedDate {
    my $self = shift;

    my $Formatter = DE_EPAGES::Core::API::Object::DateTimeFormatter->new();
    return $Formatter->format_datetime( GetCurrentDateTime(), 'date', '%d%m%Y%H%M%S' );
}

#==================================================================================================
# _parseResponse
#==================================================================================================
sub _parseResponse {
    my $self = shift;
    my ($Response) = @_;

    my $ResponseContent = $Response->decoded_content;
    Error('EMPTY_RESPONSE') unless isValid($ResponseContent);
    my @NameValueStrings = split(/&/, $ResponseContent);
    my $hParsedResponse = {};

    foreach my $NameValueString ( @NameValueStrings ) {
        my @NameValue = split(/=/, $NameValueString);
        Error('INVALID_RESPONSE_FORMAT') unless @NameValue;
        $hParsedResponse->{$NameValue[0]} = $NameValue[1];
    }

    return $hParsedResponse;
}

#==================================================================================================
# _getNumquestion
#==================================================================================================
sub _getNumquestion {
    my $self = shift;

    my $FourDigitNumber = int(rand(10000)); ## no critic qw(ProhibitMagicNumbers);
    my $LastFiveDigitsOfDate = substr(time(), -5); ## no critic qw(ProhibitMagicNumbers);

    return sprintf('%s%s', $FourDigitNumber, $LastFiveDigitsOfDate);
}

#==================================================================================================
# _getNumquestion
#==================================================================================================
sub _getMinorUnitAmount {
    my $self = shift;
    my ($Shop, $LineItemPayment) = @_;

    my $hInfoLineItemPayment = $LineItemPayment->get(['CurrencyID', 'Amount']);
    my $CurrencyID = $hInfoLineItemPayment->{'CurrencyID'};
    my $Amount = $hInfoLineItemPayment->{'Amount'};

    my @Currencies = grep {$_->{'CurrencyID'} eq $CurrencyID} @{$Shop->get('Currencies')};
    Error('INVALID_CURRENCY_ID') unless @Currencies;
    my $hCurrency = $Currencies[0];

    return $Amount * 10**$hCurrency->{'MinorUnit'}; ## no critic qw(ProhibitMagicNumbers);
}

#========================================================================================
# function     _existsObjectByGUID
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_existsObjectByGUID($GUID);
#----------------------------------------------------------------------------------------
# description  Wrapper of core binding
#----------------------------------------------------------------------------------------
# input        $GUID | general unique id of the object | string
# return       $Object | object  | object
#========================================================================================
sub _existsObjectByGUID {
    my $self = shift;
    my ($GUID) = @_;

    return ExistsObjectByGUID($GUID);
}

#========================================================================================
# function     _loadObjectByGUID
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_loadObjectByGUID($GUID);
#----------------------------------------------------------------------------------------
# description  Wrapper of core binding
#----------------------------------------------------------------------------------------
# input        $GUID | general unique id of the object | string
# return       $Object | object  | object
#========================================================================================
sub _loadObjectByGUID {
    my $self = shift;
    my ($GUID) = @_;

    return LoadObjectByGUID($GUID);
}

#========================================================================================
# function     _loadRootObject
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_loadRootObject();
#----------------------------------------------------------------------------------------
# description  Wrapper of core binding
#----------------------------------------------------------------------------------------
# return       $Object | object  | object
#========================================================================================
sub _loadRootObject {
    my $self = shift;

    return LoadRootObject();
}

sub _getWebUrlUnitySSL {
    my $self = shift;
    my ($LineItemPayment) = @_;
    my $Order = $LineItemPayment->parent()->parent();

    return $Order->get('WebUrlUnitySSL');
}

sub _addPaymentErrorCode {
    my $self = shift;
    my ($LineItemPayment, $ErrorCode) = @_;
    my $UnityErrorCode = $self->_resolveUnityErrorCode($ErrorCode);

    if (defined $ErrorCode) {
        $LineItemPayment->set({'PaymentErrorCode' => $UnityErrorCode}) 
    }
    return;
}

sub _resolveUnityErrorCode {
    my $self = shift;
    my ($ErrorCode) = @_;

    given ($ErrorCode) {
        when ('PaymentProcess_Failed') { return 'Failed'; }
        when ('PaymentProcess_Canceled') { return 'Canceled'; }
        when ('CreatePayment_Failed') { return 'Failed'; }
        when ('PaymentProcess_StatusUnknown') { return 'Failed'; }
        default { return; }
    }
}
1;
