#========================================================================================
# package      DE_EPAGES::PriceList::API::PriceList
# state        public
#----------------------------------------------------------------------------------------
# description  This package provides the main pricelist functions.
#========================================================================================
package DE_EPAGES::PriceList::API::PriceList;
use base Exporter;
use strict;

our @EXPORT_OK = qw (
    GetCustomerPriceLists
    GetProductPriceList
    IsTimeFrameDifferent
    GetUnityPriceList
);

use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::Log qw( GetLog );
use DE_EPAGES::Core::API::TestParameter qw( TestString );
use DE_EPAGES::Customer::API::Shop qw( GetCustomerForUser );
use DE_EPAGES::Object::API::Factory qw( LoadClassByAlias );
use DE_EPAGES::PriceList::API::Constants qw( UNITY_PRICELIST_PREFIX UNITY_PRICELIST_CUSTOMERGROUP_NAME );

sub ErrorDefinition {
    return {
        'NoUnityShop' => 'Shop #Shop is not a Unity shop type and therefore has no Unity price list.',
    };
}

#========================================================================================
# function     GetCustomerPriceLists
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aPriceLists = GetCustomerPriceLists( $Customer, $CurrencyID, $Shop, $User );
#----------------------------------------------------------------------------------------
# description  Returns the price list that is valid for the customer at the current time.
#               The price list may be assigned to the customer group or directly to the
#               customer.
#               Returns empty list if no price list is valid for this customer.
#----------------------------------------------------------------------------------------
# input        $Customer | Customer object | object
# input        $CurrencyID | currency id (eg. 'USD') | string
# input        $Shop | shop object (not used if customer is defined) | object
# input        $User | user object (not used if customer is defined)| object
# return       $aPriceLists | list of valid PriceList objects | ref.array.object
#========================================================================================
sub GetCustomerPriceLists($$$$) {
    my ($Customer, $CurrencyID, $Shop, $User) = @_;
    TestString( 'CurrencyID', $CurrencyID );

    if (not defined $Customer and
            defined $Shop and
            defined $User and not $User->get('IsAnonymous')) {
        $Customer = GetCustomerForUser($User, $Shop);
    }
    # try to find a pricelist via customer
    my @PriceLists = defined $Customer
        ? grep {$_->get('CurrencyID') eq $CurrencyID and $_->get('IsEffective')} @{$Customer->get('PriceLists')}
        : ();

    # try to find a pricelist via group
    my $CustomerGroup;
    if (defined $Customer) {
        $CustomerGroup = $Customer->get('CustomerGroup');
    } elsif (defined $User and not $User->get('IsAnonymous')) {
        $CustomerGroup = $Shop->get('NewMemberCustomerGroup');
    } else {
        $CustomerGroup = $Shop->get('NonMemberCustomerGroup');
    }
    if (defined $CustomerGroup) {
        push @PriceLists, grep {$_->get('CurrencyID') eq $CurrencyID and $_->get('IsEffective')} @{$CustomerGroup->get('PriceLists')};
    }
    return \@PriceLists;
}

#========================================================================================
# function     GetProductPriceList
# state        public
#----------------------------------------------------------------------------------------
# syntax       $PriceList = GetProductPriceList($Product, $Customer, $CurrencyID, $Shop, $User );
#----------------------------------------------------------------------------------------
# description  Returns the price list that is valid for the customer and product at the current time.
#               The price list may be assigned to the customer group or directly to the
#               customer.
#               Returns undef if no price list is valid for this customer.
#----------------------------------------------------------------------------------------
# input        $Product | product object | object
# input        $Customer | customer object | object
# input        $CurrencyID | currency id (eg. 'USD') | string
# input        $Shop | shop object (not used if customer is defined) | object
# input        $User | user object (not used if customer is defined)| object
# return       $PriceList | valid PriceList objects | object
#========================================================================================
sub GetProductPriceList($$$$$) {
    my ($Product, $Customer, $CurrencyID, $Shop, $User) = @_;
    foreach my $PriceList (@{GetCustomerPriceLists($Customer, $CurrencyID, $Shop, $User)}) {
        return $PriceList
            if scalar @{$PriceList->scalePrices($Product)}
            or scalar @{$PriceList->quantityDiscounts($Product)}
            or scalar @{$PriceList->valueDiscounts($Product)};
    }

    # do the next steps only for unity
    return undef unless $Shop->get('IsUnityRegistered');

    my $PriceListAlias = sprintf('%s%s', UNITY_PRICELIST_PREFIX, $CurrencyID);
    if ($Shop->folder('PriceLists')->existsChild($PriceListAlias)) {
        return $Shop->folder('PriceLists')->child($PriceListAlias);
    }

    return undef;
}

#========================================================================================
# function     IsTimeFrameDifferent
# state        public
#----------------------------------------------------------------------------------------
# syntax       $NoOverlap = IsTimeFrameDifferent($hTimeSpan1, $hTimeSpan2)
#----------------------------------------------------------------------------------------
# description  Returns true if 2 time spans do not overlap.
#----------------------------------------------------------------------------------------
# input        $hTimeSpan1 | time span with hash keys ValidFrom, ValidTo | ref.hash
# input        $hTimeSpan2 | time span with hash keys ValidFrom, ValidTo | ref.hash
# return       $NoOverlap | true if time spans do not overlap | boolean
#========================================================================================
sub IsTimeFrameDifferent {
    my ( $A,  $B) = @_;
    if (  ( $A->{'ValidFrom'} <= $B->{'ValidFrom'}    and  $B->{'ValidFrom'} <= $A->{'ValidTo'} ) or
          ( $A->{'ValidFrom'} <= $B->{'ValidTo'}      and  $B->{'ValidTo'}   <= $A->{'ValidTo'} ) or
          ( $B->{'ValidFrom'} <= $A->{'ValidFrom'}    and  $A->{'ValidFrom'} <= $B->{'ValidTo'} ) or
          ( $B->{'ValidFrom'} <= $A->{'ValidTo'}      and  $A->{'ValidTo'}   <= $B->{'ValidTo'} )
       ) {
        return 0; #there is an overlapping
    } else {
        return 1; # there is not any overlapping
    }
}

#========================================================================================
# function     GetUnityPriceList
# state        public
#----------------------------------------------------------------------------------------
# syntax       $PriceList = GetUnityPriceList($Shop, $CreateIfNotExists );
#----------------------------------------------------------------------------------------
# description  Returns the current active price list of a now shop
#               Creates a new price list if it not exists and $CreateIfNotExists is true.
#               Returns undef if no price list exists and $CreateIfNotExists is not true.
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
# input        $CreateIfNotExists | create shop pricelist if not exists (default 0)(optional) | integer
# return       $PriceList | PriceList objects | object
#========================================================================================
sub GetUnityPriceList($;$) {
    my ($Shop, $CreateIfNotExists) = @_;

    if ($Shop->get('IsUnityRegistered')) {
        my $PriceList;
        my $PriceListAlias = _getUnityPriceListAlias($Shop);
        if ($Shop->folder('PriceLists')->existsChild($PriceListAlias)) {
            $PriceList = $Shop->folder('PriceLists')->child($PriceListAlias);
        } elsif ($CreateIfNotExists) {
            my $CustomerGroup = _getOrCreateUnityCustomerGroup($Shop);
            $PriceList = _createUnityPriceList($Shop);
            $PriceList->addCustomerGroup($CustomerGroup);
        }
        return $PriceList;
    } else {
        Error('NoUnityShop', { 'Shop' => $Shop->pathString });
    }
}

sub _getUnityPriceListAlias {
    my ($Shop) = @_;
    return sprintf('%s%s', UNITY_PRICELIST_PREFIX, $Shop->get('CurrencyID'));
}

# creates a fixed pricelist for unity shops
sub _createUnityPriceList {
    my ($Shop) = @_;

    my $Folder  = $Shop->folder('PriceLists', { 'Position'=> 30 });
    my $PriceListAlias = _getUnityPriceListAlias($Shop);
    my $PriceList;
    if ($Folder->existsChild($PriceListAlias)) {
        $PriceList = $Folder->child($PriceListAlias);
    } else {
        GetLog->debug("Adding new Pricelist $PriceListAlias");
        my $Class = LoadClassByAlias('PriceList');
        my $hPriceList = {
            'Parent'     => $Folder,
            'Alias'      => $PriceListAlias,
            'ValidFrom'  => DateTime->now(),
            'ValidTo'    => DateTime->new('year'=>2999, 'month'=>1,'day'=>1),
            'CurrencyID' => $Shop->get('CurrencyID'),
        };
        $PriceList = $Class->insertObject($hPriceList);
    }
    return $PriceList;
}

# creates a fixed customergroup for unity shops
sub _getOrCreateUnityCustomerGroup {
    my ($Shop) = @_;

    my $CustomerGroups = $Shop->get('CustomerGroups');
    my $CustomerGroup;
    if (scalar @$CustomerGroups) {
        $CustomerGroup = $CustomerGroups->[0];
    } else {
        GetLog->debug("adding CustomerGroup ".UNITY_PRICELIST_CUSTOMERGROUP_NAME);
        my $Folder  = $Shop->folder('Groups', { 'Position'=> 20 });
        my $Class = LoadClassByAlias('CustomerGroup');
        my $hCustomerGroup = {
            'Parent'              => $Folder,
            'Alias'               => UNITY_PRICELIST_CUSTOMERGROUP_NAME,
            'IsNewMemberDefault'  => 1,
            'IsNonMemberDefault'  => 1,
        };
        $CustomerGroup = $Class->insertObject($hCustomerGroup);
        GetLog->debug(sprintf('Assigning Customer to CustomerGroup %s', UNITY_PRICELIST_CUSTOMERGROUP_NAME));
        # skip Customers without Users as they cannot add to the basket again
        for (@{$Shop->get('Customers')}) {
            next unless scalar @{$_->get('Users')};
            $_->set( {'CustomerGroup' => $CustomerGroup} );
        }
    }
    return $CustomerGroup;
}

1;
