#========================================================================================
# package      DE_EPAGES::Product::API::Object::PagerProducts
# state        public
# base         DE_EPAGES::Presentation::API::Object::Pager
# description  Pager for all products of a shop.
#               Allows sorting by product attributes and list price,
#========================================================================================
package DE_EPAGES::Product::API::Object::PagerProducts;
use base DE_EPAGES::Presentation::API::Object::Pager;

use strict;
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );
#========================================================================================
# function     countItems
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Count = $Pager->countItems;
#----------------------------------------------------------------------------------------
# description  Returns the total number of products in the shop.
#----------------------------------------------------------------------------------------
# return       $Count | number of items | integer
#========================================================================================
sub countItems {
    my $self = shift;
    my $Object = $self->get('Object');
    return scalar @{$Object->get('Products')};
}

#========================================================================================
# function     items
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aProducts = $Pager->items($PageNumber);
#----------------------------------------------------------------------------------------
# description  Return a list of all products on a page.
#----------------------------------------------------------------------------------------
# input        $PageNumber | current page number | integer
# return       $aProducts | list of products | ref.array.object
#========================================================================================
sub items {
    my $self = shift;
    my ($PageNumber) = @_;
    my $Object = $self->get('Object');
    my $Children = $self->sortObjects($Object->get('Products'));
    my @Items = @{$Children}[$self->firstItemOfPage($PageNumber) .. $self->lastItemOfPage($PageNumber)];
    return \@Items;
}

#========================================================================================
# function     sortObjects
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aSortedObjects = $Pager->sortObjects($aObjects);
#----------------------------------------------------------------------------------------
# description  Sorts the objects according to the 'OrderBy' and 'OrderDesc' attributes.
#               The parameter CurrencyID is required for sorting by ListPrice.
#               For sorting by attributes with value selection, add ".Value", for example
#               OrderBy => 'Size.Value'.
#               See also: <function DE_EPAGES::Product::API::Object::PagerLoopProducts::sortObjects>.
#----------------------------------------------------------------------------------------
# input        $aObjects | list of products | ref.array.object
# return       $aSortedObjects | sorted list of products  | ref.array.object
#========================================================================================
sub sortObjects {
    my $self = shift;
    my ($aObjects) = @_;

    return [] unless scalar @$aObjects;

    my @OrderBy = defined $self->orderBy ? split /\./, $self->orderBy : ();

    # use the unsorted/presorted list if OrderBy is empty
    return $aObjects unless scalar @OrderBy;

    # get values to sort, if super product without price use lowest variation price
    my $OrderBy = shift @OrderBy;
    my @Values;
    my $Type;
    my $FirstObject = $aObjects->[0];
    if( $OrderBy eq 'ListPrice' ) {
        my $TaxModel = $FirstObject->getSite()->get('TaxModel');
        my $CurrencyID = $self->get('CurrencyID');
        Error( 'NoCurrencyID' ) unless defined $CurrencyID;
        @Values = map {
            if($_->isSuperProduct()) {
                $_->getListPrice($CurrencyID, $TaxModel) //
                    $_->lowestSubProductPrice($CurrencyID, $TaxModel);
            } else {
                $_->getListPrice($CurrencyID, $TaxModel);
            }
        } @$aObjects;
        $Type = 'Money';
    }elsif( $OrderBy eq 'ManufacturerPrice' ) {
        my $TaxModel = $FirstObject->getSite()->get('TaxModel');
        my $CurrencyID = $self->get('CurrencyID');
        Error( 'NoCurrencyID' ) unless defined $CurrencyID;
        @Values = map { $_->getManufacturerPrice($CurrencyID, $TaxModel) } @$aObjects;
        $Type = 'Money';
    }else {
        @Values = map {
            my $Value = $_->getWithParent($OrderBy);
            $Value = $Value->get($_) foreach @OrderBy; # e.g. Size.Value
            $Value;
        } @$aObjects;
        $Type = $FirstObject->class->attribute( $OrderBy )->get('Type');
    }

    # start with undefined values
    my @SortedKeys = grep { not defined $Values[$_] } (0..scalar(@Values)-1);

    # sort different on different types
    if (ElementOf($Type, qw (Integer Float Boolean Money))) {
        push @SortedKeys, sort {$Values[$a] <=> $Values[$b]} grep { defined $Values[$_] } (0..scalar(@Values)-1);
    } else {
        push @SortedKeys, sort {uc $Values[$a] cmp uc $Values[$b]} grep { defined $Values[$_] } (0..scalar(@Values)-1);
    }
    my @Objects = @$aObjects[@SortedKeys];

    # sort descending ?
    @Objects = reverse @Objects if $self->orderDesc;

    return \@Objects;
}

1;
