#========================================================================================
# package      DE_EPAGES::ShopMigration::API::JSONLines::ShopObjectImportHandler
# state        public
#----------------------------------------------------------------------------------------
# description  Base class for json import drivers based on a shop
#========================================================================================
package DE_EPAGES::ShopMigration::API::JSONLines::ShopObjectImportHandler;
use base DE_EPAGES::ShopMigration::API::JSONLines::BaseImportHandler;

use strict;
use DE_EPAGES::Core::API::Error qw(Error ExistsError VarsOnError);
use File::Spec::Functions;

#ERRORDEFINITION
sub ErrorDefinition {
    return {
        'JSON_SHOP_UNDEFINED' => 'Hash element Shop is mandatory for this function call',
    };
}
#ERRORDEFINITION

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Importer = DE_EPAGES::ShopMigration::API::JSONLines::ShopObjectImportHandler->new(%Options);
#----------------------------------------------------------------------------------------
# description  create a new json import handler
#----------------------------------------------------------------------------------------
# input        %Options  | shop and dir name | hash string
# return       $Importer |  importer    | object
#========================================================================================
sub new {
    my $class = shift;
    my (%Options) = @_;

    my ($Shop, $DirName);
    if (exists $Options{Shop}) {
        $Shop = $Options{Shop};
        delete $Options{Shop};
    }
    if (exists $Options{DirName}) {
        $DirName = $Options{DirName};
        delete $Options{DirName};
    }
    $DirName //= '.';
    Error('JSON_SHOP_UNDEFINED') unless defined $Shop;
    my $self = $class->SUPER::new(%Options);
    $self->shop($Shop);
    $self->isMigrationPrepShop($Shop->get('IsMigrationPrepShop'));
    #add shop alias to dirname
    if ($self->isMigrationPrepShop) {
        $self->dirName(catdir($DirName, $Shop->get('MigrationShop')->alias));
        $self->baseFileName('base2prep_' . $self->defaultBaseFileName());
    } else {
        $self->dirName(catdir($DirName, $Shop->alias));
        $self->baseFileName('prep2base_' . $self->defaultBaseFileName());
    }
    return $self;
}

#========================================================================================
# function     shop
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->shop;
# syntax       $self->shop($Shop);
#----------------------------------------------------------------------------------------
# description  returns or sets the shop object for the exporter (used in subclasses)
#----------------------------------------------------------------------------------------
# input        $Shop | new shop | object
# return       $FileName | file name | string
#========================================================================================
sub shop {
    my $self = shift;
    my ($Shop) = @_;
    if (@_ == 1) {
        $self->{Shop} = $Shop;
    }
    return $self->{Shop};
}

#========================================================================================
# function     isMigrationPrepShop
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->isMigrationPrepShop;
# syntax       $self->isMigrationPrepShop(0);
#----------------------------------------------------------------------------------------
# description  returns or sets the isMigrationPrepShop field
#----------------------------------------------------------------------------------------
# input        $Value | isMigrationPrepShop | boolean
# return       $FileName | file name | string
#========================================================================================
sub isMigrationPrepShop {
    my $self = shift;
    my ($Value) = @_;
    if (@_ == 1) {
        $self->{IsMigrationPrepShop} = $Value;
    }
    return $self->{IsMigrationPrepShop};
}

#========================================================================================
# function     importLine
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->importLine();
#----------------------------------------------------------------------------------------
# description  imports one data line, get the line element from function topID('Item')
#               this is only a dummy funtion here.
#========================================================================================
sub importLine {
    my $self = shift;

    my $hFields = $self->topID('Item');
    eval {
        if ($self->isMigrationPrepShop()) {
            $self->setShopDataPrepShop($hFields);
        } else {
            $self->setShopDataBaseShop($hFields);
        }
    };
    VarsOnError({'LineNumber'=> $self->lineNumber }) if ExistsError();
    return;
}

#========================================================================================
# function     setShopDataPrepShop (abstract)
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->setShopDataPrepShop();
#----------------------------------------------------------------------------------------
# description  sets the imported data fields for the prep shop
#========================================================================================
sub setShopDataPrepShop {
    my $self = shift;
    die 'please implement this function';
}

#========================================================================================
# function     setShopDataBaseShop (abstract)
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->setShopDataBaseShop();
#----------------------------------------------------------------------------------------
# description  sets the imported data fields for the base shop
#========================================================================================
sub setShopDataBaseShop {
    my $self = shift;
    die 'please implement this function';
}

1;
