use strict;
use Getopt::Long;
use Time::HiRes qw(time);
use DE_EPAGES::Core::API::Script qw(RunScript);
use DE_EPAGES::Core::API::Error qw(ExistsError GetError);
use DE_EPAGES::Core::API::Warning qw(Warning);
use DE_EPAGES::Database::API::Connection qw(RunOnStore GetCurrentDBHandle);
use DE_EPAGES::Object::API::Factory qw(LoadRootObject LoadObject LoadObjectByPath);
use DE_EPAGES::WebInterface::API::MessageCenter qw(SynchronizeCache);

RunScript(Sub => \&Main, Debug => 1);

sub Main {

    my %Options = ();
    my @GetOpt = qw(help passwd=s storename=s feature=s max=s setto=s prepshops shoptype=s);
    Usage("Unkown parameter specified.\n") unless GetOptions(\%Options, @GetOpt);
    Usage() if $Options{help};
    for (qw(storename feature max)) {
        Usage(printf("Missing required parameter -%s\n", $_)) unless defined $Options{$_};
    }

    my $time1 = time;
    RunOnStore(
        Store      => $Options{storename},
        DBPassword => $Options{passwd},
        Sub        => sub {
            FixShopsWithFeature(\%Options);
        }
    );
    my $time2 = time;
    printf "need %.1f seconds\n", $time2 - $time1;
    eval {SynchronizeCache();};
    Warning(GetError()) if ExistsError();
}

sub FixShopsWithFeature {
    my ($Options) = @_;

    my $Feature = $Options->{feature};
    my $MaxValue = $Options->{max};
    my $SetTo = $Options->{setto};
    my $hShopIDs = getShops($Options);
    foreach my $ShopID (keys %$hShopIDs) {
        my $Shop = LoadObject($ShopID);
        my $BaseShopAlias = $hShopIDs->{$ShopID};
        $BaseShopAlias =~ s/prepshop//g;
        my $BaseShop = LoadRootObject()->folder('Shops')->child($BaseShopAlias);
        my $CurrentFeatureValue = $Shop->featureMaxValue($Feature);
        if ($CurrentFeatureValue <= $MaxValue) {
            if (defined $SetTo) {
                $Shop->featureMaxValue($Feature, $SetTo);
                printf "Shop %s (%s) feature %s changed from %s to %s\n",
                    $Shop->alias, $ShopID, $Feature, $CurrentFeatureValue, $SetTo;
            } else {
                printf "Shop %s (%s) has %s with maxvalue %s (Used in Prep: %s | Used in Base: %s)!\n",
                    $Shop->alias, $ShopID, $Feature, $CurrentFeatureValue,
                    $Shop->featureCurrentValue($Feature),
                    defined $BaseShop ? $BaseShop->featureCurrentValue($Feature) : '';
            }
        }
    }
}

sub getShops {
    my ($Options) = @_;

    my $dbi = GetCurrentDBHandle();
    my $ShopFolderID = LoadRootObject()->child('PrepShops')->id;
    my @WhereClauses;
    if (defined $Options->{prepshops} and $Options->{prepshops}) {
        push @WhereClauses, "o.parentid = $ShopFolderID";
    }
    if (defined $Options->{shoptype} and $Options->{shoptype}) {
        my @ShopTypes = split(/,/, $Options->{shoptype});
        my @ShopTypeIDs;
        foreach my $ShopTypeAlias (@ShopTypes) {
            push @ShopTypeIDs, LoadObjectByPath('/ShopTypeRefs')->child($ShopTypeAlias)->id;
        }
        push @WhereClauses, sprintf('s.shoptyperefid IN (%s)', join(',', @ShopTypeIDs));
    }

    my $sql = 'select s.shopid, o.alias from shop s inner join object o on o.objectid = s.shopid';
    $sql .= scalar @WhereClauses ? (' WHERE ' . join(' AND ', @WhereClauses)) : '';

    my $aaResult = $dbi->execute($sql, [], [ qw(Integer String) ]);
    my %ShopAlias = ();
    foreach my $Row (@$aaResult) {
        $ShopAlias{ $Row->[0] } = $Row->[1];
    }
    my $Count = keys %ShopAlias;
    print "Found $Count shops total.\n";
    return \%ShopAlias;
}




sub Usage {
    print <<END_USAGE;
Usage:
    perl $0 [options] [flags]

options:
    -passwd     database user password                                     ()
    -storename  name of store                                       (required)
    -feature    FeatureName to check                                (required)
    -max        check if featureMaxValue is below/equal this value  (required)
    -setto      if below/equal max set this new maxvalue            (optional)
    -prepshops  only check prepshops                                (optional)
    -shoptypes  check all shops of this shoptype(s)                 (optional)

flags:
    -help       show the command line options

Example:
    perl $0 -storename Store  -storename Store -feature Languages -max 14 -shoptype MigrationNowL,MigrationNowXL
    perl $0 -storename Store  -storename Store -feature Languages -max 1 -shoptype MigrationNowM
    perl $0 -storename Store  -storename Store -feature Languages -max 1 -setto 15
END_USAGE
    exit 2;
}


