use strict;
use File::Spec::Functions;
use Getopt::Long;
use String::Random;
use Time::HiRes qw(time);
use 5.010; # Needed because we use state vars

use DE_EPAGES::Cartridge::API::Table::Cartridge qw(ExistsCartridgeByPackage);
use DE_EPAGES::Core::API::Error qw(ExistsError GetError Error);
use DE_EPAGES::Core::API::File qw(WriteFile DeleteFile);
use DE_EPAGES::Core::API::Log qw(LogError LogContext);
use DE_EPAGES::Core::API::Script qw(RunScript);
use DE_EPAGES::Core::API::Warning qw(Warning);
use DE_EPAGES::Database::API::Connection qw(RunOnStore GetCurrentDBHandle);
use DE_EPAGES::Object::API::Factory qw(LoadRootObject);
use DE_EPAGES::ShopMigration::API::Constants qw(SHOPMIGRATION_INITIAL_ALLOW
    SHOPMIGRATION_INITAL_FAILED SHOPMIGRATION_FINAL_FAILED
    SHOPMIGRATION_INITIAL_IN_PROGRESS SHOPMIGRATION_FINAL_ALLOW SHOPMIGRATION_ABORTED);
use DE_EPAGES::ShopMigration::API::MigrationShop qw(
    ExistsPrepShop
    FindAllShopsWithMigrationStatus CreatePrepShop
    ExportShopDataForMigration ImportShopDataForMigration
    PreMigrationImportTasks PostMigrationImportTasks);
use DE_EPAGES::WebInterface::API::MessageCenter qw(SynchronizeCache);
use File::Basename qw(basename);

##no critic qw( RequireCommentForExportedFunctions ProhibitMultiplePackages RequireArgUnpacking ProhibitPackageVars )
# hack log4perl / cspec.E and cspec.G for easier search in ELK
use DE_EPAGES::WebInterface::API::ApplicationServer;
package DE_EPAGES::WebInterface::API::ApplicationServer;
sub currentRequest {return shift;}
sub headers {return shift;}
sub address {return '127.0.0.1';}
sub port {return '0';}
sub header {return @_[0]->{@_[1]};}
our $FakeAppSrv = bless {}, 'DE_EPAGES::WebInterface::API::ApplicationServer';
$DE_EPAGES::WebInterface::API::ApplicationServer::APPLICATION_SERVER = $FakeAppSrv;
# end of hack
## use critic

package main; ##no critic ( ProhibitMultiplePackages )

sub Main {
    local $| = 1;
    my %Options = ();

    my @GetOpt = qw(help passwd=s storename=s shop=s directory=s onlyimport prep! final! aborted! output!
        printfailed resetfailed maxminutes=s mailto=s mailfrom=s);
    use constant TWENTY_FOUR_HOURS => 1440;
    $Options{maxminutes} = TWENTY_FOUR_HOURS;
    $Options{prep} = 1;
    $Options{final} = 1;
    $Options{aborted} = 1;
    $Options{errmailcontent} = '';
    usage("Unkown parameter specified.\n") unless GetOptions(\%Options, @GetOpt);
    usage() if $Options{help};
    usage('Missing parameter -storename') unless $Options{storename};
    $Options{directory} //= catdir($ENV{EPAGES_WEBSECURE}, 'ShopExport', $Options{storename});

    my $time1 = time;
    my $hCount = {};
    RunOnStore(
        Store      => $Options{storename},
        DBPassword => $Options{passwd},
        Sub        => sub {
            return unless (ExistsCartridgeByPackage('DE_EPAGES::ShopMigration'));
            if ($Options{printfailed}) {
                printFailed();
            } elsif ($Options{resetfailed}) {
                resetFailed(\%Options);
            } else {
                $Options{MaxDateTime} = GetCurrentDBHandle()->currentDateTime->add('minutes' => $Options{maxminutes});
                if ($Options{prep}) {
                    $Options{Action} = 'Initial';
                    $Options{WantedStatus} = SHOPMIGRATION_INITIAL_IN_PROGRESS;
                    $Options{WantedString} = 'INITIAL_IN_PROGRESS';
                    $hCount->{CreatedPrepShops} = migrateShops(\%Options);
                }
                if ($Options{final}) {
                    $Options{Action} = 'Final';
                    $Options{WantedStatus} = SHOPMIGRATION_FINAL_ALLOW;
                    $Options{WantedString} = 'FINAL_ALLOW';
                    $hCount->{FinalMigration} = migrateShops(\%Options);
                }
                deleteAborted(\%Options) if ($Options{aborted});
            }
        },
    );
    my $time2 = time;
    foreach my $Type (sort keys %$hCount) {
        printf "\n# Statistics (%s) #\n", $Type if $Options{output};
        $hCount->{$Type}{Left} = $hCount->{$Type}{Total} - $hCount->{$Type}{Done} - $hCount->{$Type}{Failed};
        printf join('', map {"%-13s: %s\n"} keys %{$hCount->{$Type}}),
            'Total Shops', $hCount->{$Type}{Total},
            'Succesful', $hCount->{$Type}{Done},
            'Failed', $hCount->{$Type}{Failed},
            'Left', $hCount->{$Type}{Left} if $Options{output};
    }
    printf "need %.3f seconds\n", $time2 - $time1 if $Options{output};
    sendErrorEMail(\%Options);
    eval {SynchronizeCache();};
    Warning(GetError()) if ExistsError();
}

sub sendErrorEMail {
    my ($Options) = @_;

    return unless $Options->{errmailcontent};
    return unless $Options->{mailto};
    my $Header = 'One or multiple shops failed to migrate:';
    my $Content .= sprintf "%s\n\n%s", $Header, $Options->{errmailcontent};
    my $ScriptName = basename($0);
    if ($ScriptName =~ /(.*)\.pl/) {$ScriptName = $1;}
    my $Hostname = Sys::Hostname::hostname();
    $Hostname =~ s/\..*//;
    my $TmpFile = catfile(File::Spec->tmpdir(), "migrateShops.mail.$$");
    WriteFile($TmpFile, \$Content);
    my $syscall
        = catfile($ENV{EPAGES_PERL}, 'bin', 'perl') . ' '
        . catfile($ENV{EPAGES_CARTRIDGES}, 'DE_EPAGES', 'Core', 'bin', 'SendProgramOutput.pl')
        . sprintf(' -to=%s -from=%s -subject="%s: Output from program %s"  -ini /srv/epages/eproot/Shared/Config/Scheduler.conf -file=%s',
        $Options->{mailto}, $Options->{mailfrom}, $Hostname, $ScriptName, $TmpFile);
    printf "<<< %s\n", $syscall;
    system($syscall);
    DeleteFile($TmpFile);
}

sub printFailed {
    my $aFailedPrep = FindAllShopsWithMigrationStatus(SHOPMIGRATION_INITAL_FAILED);
    my $aFailedFinal = FindAllShopsWithMigrationStatus(SHOPMIGRATION_FINAL_FAILED);
    if (scalar @$aFailedPrep) {
        printf "Shops in state 'SHOPMIGRATION_INITAL_FAILED':\n%s\n",
            join("\n", map {$_->alias} @$aFailedPrep);
    }
    if (scalar @$aFailedFinal) {
        printf "Shops in state 'SHOPMIGRATION_FINAL_FAILED':\n%s\n",
            join("\n", map {$_->alias} @$aFailedFinal);
    }
    if (!scalar @$aFailedPrep && !scalar @$aFailedFinal) {
        printf "Found no shops in failed migration status in this store.\n",
    }
    return;
}

sub resetFailed {
    my ($hOptions) = @_;

    my $aFailedPrep = FindAllShopsWithMigrationStatus(SHOPMIGRATION_INITAL_FAILED);
    my $aFailedFinal = FindAllShopsWithMigrationStatus(SHOPMIGRATION_FINAL_FAILED);
    my ($ShopAlias, $Output) = @{$hOptions}{qw(shop output)};
    if (defined $ShopAlias) {
        $aFailedPrep = [ grep {$_->alias eq $ShopAlias} @$aFailedPrep ];
        $aFailedFinal = [ grep {$_->alias eq $ShopAlias} @$aFailedFinal ];
    }
    if (scalar @$aFailedPrep) {
        printf "Resetting shops with 'SHOPMIGRATION_INITAL_FAILED':\n" if $Output;
        for (@$aFailedPrep) {
            my $PrepShop = $_->get('MigrationPrepShop');
            my $DeletedPrepShop = '';
            if ($PrepShop) {
                $DeletedPrepShop = sprintf(' (also PrepShop %s deleted)', $PrepShop->alias) if $Output;
                $PrepShop->delete;
            }
            $_->set({ MigrationStatus => SHOPMIGRATION_INITIAL_IN_PROGRESS });
            printf "%s%s\n", $_->alias, $DeletedPrepShop if $Output;
        }
    }
    if (scalar @$aFailedFinal) {
        printf "Resetting shops with 'SHOPMIGRATION_FINAL_FAILED':\n";
        for (@$aFailedFinal) {
            $_->set({ MigrationStatus => SHOPMIGRATION_FINAL_ALLOW });
            printf "%s\n", $_->alias if $Output;
        }
    }
    if (!scalar @$aFailedPrep && !scalar @$aFailedFinal) {
        printf "No Shop in failed status found!\n" if $Output;
    }
    return;
}

sub deleteAborted {
    my ($hOptions) = @_;

    my $aAborted = FindAllShopsWithMigrationStatus(SHOPMIGRATION_ABORTED);
    my ($ShopAlias, $Output) = @{$hOptions}{qw(shop output)};
    if (defined $ShopAlias) {
        $aAborted = [ grep {$_->alias eq $ShopAlias} @$aAborted ];
    }
    if (scalar @$aAborted) {
        printf "Resetting status for shops with status 'SHOPMIGRATION_ABORTED':\n" if $Output;
        for (@$aAborted) {
            my $PrepShop = $_->get('MigrationPrepShop');
            my $DeletedPrepShop = '';
            if ($PrepShop) {
                $DeletedPrepShop = sprintf(' (also PrepShop %s deleted)', $PrepShop->alias) if $Output;
                $PrepShop->delete;
            }
            $_->set({ MigrationStatus => SHOPMIGRATION_INITIAL_ALLOW });
            printf "%s%s\n", $_->alias, $DeletedPrepShop if $Output;
        }
    }
    if (!scalar @$aAborted) {
        printf("No shops in status '%s' (%s) found!\n", SHOPMIGRATION_ABORTED, 'ABORTED') if $Output;
    }
    return;
}

sub migrateShops {
    my ($hOptions) = @_;

    my %Count = (Total => 0, Failed => 0, Done => 0);
    my ($StoreName, $Directory, $OnlyImport, $ShopAlias, $Output) =
        @{$hOptions}{qw(storename directory onlyimport shop output)};
    my $WantedStatus = $hOptions->{WantedStatus};
    my $WantedString = $hOptions->{WantedString};
    my $aShopList = defined $ShopAlias ?
        findSingleShop($ShopAlias, $WantedStatus, $WantedString) :
        FindAllShopsWithMigrationStatus($WantedStatus);
    $Count{Total} = scalar @$aShopList;
    unless ($Count{Total}) {
        printf("No shops in status '%s' (%s) found!\n", $WantedStatus, $WantedString) if $Output;
        return \%Count;
    }
    printf "Found %s shops in status %s\n", $Count{Total}, $WantedString if $Output;
    foreach my $Shop (@$aShopList) {
        last if (GetCurrentDBHandle()->currentDateTime > $hOptions->{MaxDateTime});
        eval {
            my ($SiteID, $RequestID) = ($Shop->get('GUID'), generateRequestID());
            $FakeAppSrv->{'x-epages-requestid'} = $RequestID;
            $FakeAppSrv->{'x-epages-site'} = $SiteID;
            LogContext({ 'Shop' => $Shop->alias, 'RequestID' => $RequestID }, sub {
                if ($WantedStatus == SHOPMIGRATION_FINAL_ALLOW) {
                    Error('HasNoSSLCertifcate') if (defined $Shop->get('DomainName') && !$Shop->get('HasSSLCertificate'));
                }
                my $SourceShop = $hOptions->{Action} eq 'Initial' ? $Shop : $Shop->get('MigrationPrepShop');
                ExportShopDataForMigration($SourceShop, $Directory);
                PreMigrationImportTasks($Shop);
                my $TargetShop = $hOptions->{Action} eq 'Initial' ?
                    ($OnlyImport ? $Shop->get('MigrationPrepShop') : CreatePrepShop($Shop)) : $Shop;
                ImportShopDataForMigration($TargetShop, $Directory);
                PostMigrationImportTasks($Shop);
            });
        };
        # Set error state on BaseShop in case something goes wrong so we don't try over and over
        if (ExistsError()) {
            my $Error = GetError();
            if ($WantedStatus == SHOPMIGRATION_INITIAL_IN_PROGRESS) {
                $Shop->set({ 'MigrationStatus' => SHOPMIGRATION_INITAL_FAILED });
                my $PrepShop = $Shop->get('MigrationPrepShop');
                $PrepShop->delete if (defined $PrepShop);
            } else {
                $Shop->set({ 'MigrationStatus' => SHOPMIGRATION_FINAL_FAILED, 'IsClosed' => 0 });
            }
            my $Msg = sprintf "%s Migration for shop %s store %s failed with '%s'\n",
                $hOptions->{Action}, $Shop->alias, $StoreName, $Error->shortMessage;
            $hOptions->{errmailcontent} .= $Msg . "\n";
            print $Msg;
            LogError($Error);
            $Count{Failed}++;
        } else {
            printf "%s Migration for shop %s store %s succeeded\n",
                $hOptions->{Action}, $Shop->alias, $StoreName;
            $Count{Done}++;
        }
    }
    return \%Count;
}

sub initializeGenerator {
    my $Generator = String::Random->new;
    $Generator->{'A'} = [ @{$Generator->{'C'}}, @{$Generator->{'c'}}, @{$Generator->{'n'}} ];
    return $Generator;
}

# Generates a RequestID of 27 chars which looks like the Apache unique_id
sub generateRequestID {
    state $Generator = initializeGenerator();
    return $Generator->randpattern('AAAAAAAAAAAAAAAAAAAAAAAAAAA');
}

sub findSingleShop {
    my ($ShopAlias, $WantedStatus, $WantedString) = @_;

    my $ShopFolder = LoadRootObject()->child('Shops');
    usage(sprintf('No such shop: %s', $ShopAlias)) unless $ShopFolder->existsChild($ShopAlias);
    if (!$ShopFolder->existsChild($ShopAlias)) {
        printf("Shop is not existing: %s\n", $ShopAlias);
        exit 2;
    }
    my $Shop = $ShopFolder->child($ShopAlias);
    my ($IsStatus, $IsStatusString) = @{$Shop->get([ 'MigrationStatus', 'MigrationStatusString' ])}
        {qw(MigrationStatus MigrationStatusString)};
    if ($IsStatus != $WantedStatus) {
        printf("Shop %s is not in the proper status (Wanted: %s [%s], Is: %s [%s])\n",
            $ShopAlias, $WantedStatus, $WantedString, $IsStatus, $IsStatusString);
        return [];
    } elsif (!$Shop->get('IsMigrationShop')) {
        printf("Shop is not marked for migration (IsMigrationShop=0)\n", $ShopAlias);
        exit 2;
    }
    return [ $Shop ];
}

sub usage {
    my ($ErrorNote) = @_;

    print STDERR "\n$ErrorNote\n\n" if $ErrorNote;
    print <<END_USAGE;
Migrate one or multiple (accordingly flagged) shops to either prep stage (prepshops)
or final stage (transfer shops).

Usage:
    perl $0 [options] flags objects

options:
    -passwd      database user password                                    ()
    -shop        only use this shop alias instead of all matching shops    ()
    -directory   output directory                                          (\$EPAGES_WEBSECURE/ShopExport/<Store>)
    -storename   name of store                                             required
    -maxminutes  maximum runtime in minutes                                (1440)
    -mailto      send an additional mail in case of an error               (optional)
    -mailfrom    sender address                                            (optional)

flags:
    -help        show the command line options                             ()
    -(no)prep    migrate fitting shop(s) to prep stage                     (1)
    -(no)final   transfer fitting shop(s) to final stage                   (1)
    -(no)aborted delete prepshops of shops with status aborted             (1)
    -onlyimport  do not create a new prepshop, only import                 (0)
    -(no)output  turn on more verbose status output                        (0)
    -printfailed list shop(s) with failed migration status (will do nothing else)
    -resetfailed reset shop(s) with failed migration status (to try again, will do nothing else)

Example:
    perl $0 -storename Store -shop DemoShop -directory /tmp/shopexport
    perl $0 -storename Store -shop DemoShop -directory /tmp/shopexport -prep
    perl $0 -storename Store -shop DemoShop -directory /tmp/shopexport -final
    perl $0 -storename Store -prep -nofinal -noaborted -output
    perl $0 -storename Store -output
    perl $0 -storename Store -prep -final -aborted -mailto someone\@example.com -mailfrom epages\@example.com
END_USAGE
    exit 2;
}

RunScript(
    'Sub' => \&Main,
);

1;
