#==============================================================================
# package      DE_EPAGES::Tax::UI::Shop
# state        private
#==============================================================================
package DE_EPAGES::Tax::UI::Shop;
use base DE_EPAGES::Presentation::UI::Object;

use strict;
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError ResetError );
use DE_EPAGES::Core::API::PerlTools qw( GetCurrentDateTime );
use DE_EPAGES::Core::API::Warning qw ( Warning );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject LoadClassByAlias ExistsClassByAlias );
use DE_EPAGES::Object::API::Table::Country qw (GetInfoCountry);
use DE_EPAGES::Tax::API::Table::TaxRate qw ( ExistsTaxRate GetInfoTaxRate InsertTaxRate);## no critic qw(RequireCorrectImports)
use DE_EPAGES::Tax::API::Table::TaxRateDigital qw ( ExistsTaxRateDigital GetInfoTaxRateDigital InsertTaxRateDigital);## no critic qw(RequireCorrectImports)
use DE_EPAGES::Tax::API::Shop qw ( GetTaxMethodsFolder EnableShopTaxMethod DisableShopTaxMethod );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );
use DE_EPAGES::Core::API::TestParameter qw ( TestObject );

use constant POSITION_DEFAULT => 9999;
use constant PERCENT => 100.00;

#ERRORDEFINITION
sub ErrorDefinition {
    return {
        'NoTaxMatrix' => 'The object (#Path) is not a tax matrix.',
        'TaxMatrixAlreadyCustomized' => 'The tax matrix of shop #ShopAlias is already customized.',
        'TaxMatrixAlreadyExists' => 'The tax matrix #TaxMatrixAlias already exists in shop #ShopAlias.',
    }
}
#ERRORDEFINITION END

#==============================================================================
# Event handlers
#==============================================================================
sub ViewTaxMatrix {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop   = $Servlet->object;
    my $Matrix = $Shop->get('TaxMatrix');

    my $SystemTaxMatrix = LoadRootObject()->child( $Matrix->alias );
    if($self->NewTaxAreaAdded($SystemTaxMatrix, $Matrix) || $self->NewTaxClassAdded($SystemTaxMatrix, $Matrix)) {
        $Servlet->params('ShowResetButton', 0);
    }
    else {
        $Servlet->params('ShowResetButton', 1);
    }

    # show always the reset button because we can not add classes or areas
    $Servlet->params('ShowResetButtonDigital', 1);

    foreach my $MatrixName (qw(TaxMatrix TaxMatrixDigital)) {

        my $TaxMatrix = $Shop->get($MatrixName);
        return unless defined $TaxMatrix;
        # use a postfix for having the normal or digital tax matrix
        my $Postfix = $TaxMatrix->instanceOf('TaxMatrixDigital') ? 'Digital' : '';
        my $NameTaxAreaID  = 'TaxArea'.$Postfix.'ID';
        my $NameTaxClassID = 'TaxClass'.$Postfix.'ID';
        my $NameTaxRate    = 'TaxRate'.$Postfix;
        my $NameTaxClasses = 'TaxClasses'.$Postfix;
        my $NameTaxAreas   = 'TaxAreas'.$Postfix;

        # collect names, ordered by Position and Alias
        my $aTaxClasses = $TaxMatrix->get('TaxClasses'.$Postfix);
        my $aTaxAreas   = $TaxMatrix->get('TaxAreas'.$Postfix);

        my @TaxAreas;
        my $Counter = 0;
        foreach my $TaxArea (@$aTaxAreas) {
            my $Info = $TaxArea->tleHash;
            # get native names
            $Info->{'Country'} = GetInfoCountry($TaxArea->get('CountryID')) if $Postfix eq 'Digital';
            my @TaxClasses;

            foreach my $TaxClass (@$aTaxClasses) {
                my $TaxClassHash = $TaxClass->tleHash;
                ## no critic qw(ProhibitNoStrict)
                no strict 'refs';
                &{"InsertTaxRate".$Postfix}({$NameTaxAreaID => $TaxArea->id, $NameTaxClassID =>$TaxClass->id, 'Rate'=>0})
                unless &{"ExistsTaxRate".$Postfix}($TaxArea->id, $TaxClass->id);

                # 4 digits for e.g. 0.15 % rate
                $TaxClassHash->{$NameTaxRate} = sprintf ('%0.4f', (PERCENT * &{"GetInfoTaxRate".$Postfix}($TaxArea->id, $TaxClass->id)->{'Rate'})) ;
                use strict 'refs';
                ## use critic
                $TaxClassHash->{'ErrorItemNo'} = $Counter++;
                push @TaxClasses, $TaxClassHash;
            }
            $Info->{$NameTaxClasses} = \@TaxClasses;
            push @TaxAreas, $Info;
        }
        $Servlet->params($NameTaxAreas, \@TaxAreas);
        $Servlet->params($NameTaxClasses, [ map { $_->tleHash } @$aTaxClasses] );

    }

    return;
}

sub NewTaxMatrix {
    my $self = shift;
    my ($Servlet) = @_;

    return $self->_newTaxMatrix($Servlet);
}

sub NewTaxMatrixDigital {
    my $self = shift;
    my ($Servlet) = @_;

    return $self->_newTaxMatrix($Servlet,1);
}

sub _newTaxMatrix {
    my $self = shift;
    my ($Servlet,$IsDigital) = @_;

    my $Postfix = $IsDigital ? 'Digital' : '';
    my $MatrixAlias = $Servlet->params('TaxMatrix'.$Postfix);

    my $System = LoadRootObject();
    my $SystemTaxMatrix = $System->child( $MatrixAlias );
    Error( 'NoTaxMatrix'.$Postfix => { Path => $SystemTaxMatrix->get('Path') } )
        unless $SystemTaxMatrix->instanceOf( 'TaxMatrix'.$Postfix );

    my $Shop = $Servlet->object;

    # test if the shop already has a customized tax matrix
    if( $Shop->get('TaxMatrix'.$Postfix)->getSite->id == $Shop->id ) {
        Warning( 'TaxMatrixAlreadyCustomized', {
            'ShopAlias' => $Shop->alias,
        });
        return;
    }

    if ( $Shop->existsChild($MatrixAlias) ) {
        # The system tax matrix was aready cloned to the shop, but the customization
        # did not complete successfully, as the new tax matrix is not yet set as
        # shop attribute.
        # Maybe the customization process is still running. In this case the error
        # will disappear when the process finishes.
        Error( 'TaxMatrixAlreadyExists', {
            'ShopAlias' => $Shop->alias,
            'TaxMatrixAlias' => $MatrixAlias,
        });
        return;
    }

    # no transaction, because this can take a very long time...
    my $NewMatrix = $SystemTaxMatrix->clone({
        'Parent' => $Shop,
        'CreationDate' => GetCurrentDateTime()
    }, 1);
    $Shop->set({ 'IsClosed' => 1 });
    eval { $Shop->set({ 'TaxMatrix'.$Postfix => $NewMatrix }) };
    my $Error = ExistsError() ? GetError() : undef;
    $Shop->set({ 'IsClosed' => 0 });
    Error( $Error ) if defined $Error;

    # correct defaults for taxareadigital and taxclassdigital on Shop
    # Attributes must point to shop tax matrix digital
    $self->_correctDefaults($Shop,$NewMatrix) if $IsDigital;

    # only for normal editable tax matrix we change the area countries
    unless ($IsDigital) {
        my $ahTaxAreaCountries = $SystemTaxMatrix->taxAreaCountries();
        my $aNewTaxAreas = $NewMatrix->get('TaxAreas');
        foreach my $hOldTaxAreaCountry (@{$ahTaxAreaCountries}) {
            my $OldTaxArea = $System->loadObject($hOldTaxAreaCountry->{'TaxAreaID'}, 'TaxArea');
            foreach my $NewTaxArea (@{$aNewTaxAreas}) {
                next unless $OldTaxArea->alias eq $NewTaxArea->alias;
                $NewTaxArea->addCountry(
                    $hOldTaxAreaCountry->{'CountryID'},
                    $hOldTaxAreaCountry->{'ZipcodeFrom'},
                    $hOldTaxAreaCountry->{'ZipcodeTo'},
                    $hOldTaxAreaCountry->{'RegionName'}
                );
            }
        }
    }

    return;
}

sub ResetTaxMatrix {
    my $self = shift;
    my ($Servlet) = @_;

    return $self->_resetTaxMatrix($Servlet);
}

sub ResetTaxMatrixDigital {
    my $self = shift;
    my ($Servlet) = @_;

    return $self->_resetTaxMatrix($Servlet,1);
}

sub _resetTaxMatrix {
    my $self = shift;
    my ($Servlet,$IsDigital) = @_;

    my $Postfix = $IsDigital ? 'Digital' : '';
    my $MatrixAlias = $Servlet->params('TaxMatrix'.$Postfix);

    my $SystemTaxMatrix = LoadRootObject()->child( $MatrixAlias );
    Error( 'NoTaxMatrix'.$Postfix => { Path => $SystemTaxMatrix->get('Path') } )
        unless $SystemTaxMatrix->instanceOf( 'TaxMatrix'.$Postfix );

    my $Shop = $Servlet->object;
    my $Matrix = $Shop->get('TaxMatrix'.$Postfix);

    # if the user reload the reseted matrix page the new system matrix should not be deleted
    Error('INVALID_FORM', {
        'ViewVars'=> {
            'FormError' => 0,
        },
        'ViewAction' => 'ViewTaxMatrix',
    }) if ($SystemTaxMatrix->id == $Matrix->id);


    # only for normal editable tax matrix we test if a new tax class or area was added
    unless ($IsDigital) {
        if($self->NewTaxAreaAdded($SystemTaxMatrix, $Matrix)) {
            $Servlet->form->executeFormError({
                'Name' => 'TaxAreaAdded',
                'Reason' => 'TaxAreaAdded',
            });
        }
        if($self->NewTaxClassAdded($SystemTaxMatrix, $Matrix)) {
            $Servlet->form->executeFormError({
                'Name' => 'TaxClassAdded',
                'Reason' => 'TaxClassAdded',
            });
        }
    }

    # close shop, set matrix to system matrix and open shop
    $Shop->set({ 'IsClosed' => 1 });
    eval {
        $Shop->set({ 'TaxMatrix'.$Postfix => $SystemTaxMatrix });
        $self->_correctDefaults($Shop,$SystemTaxMatrix) if $IsDigital;
        $Matrix->delete();
    };
    my $Error = ExistsError() ? GetError() : undef;
    $Shop->set({ 'IsClosed' => 0 });
    Error( $Error ) if defined $Error;

    return;
}

sub _correctDefaults {
    my $self = shift;
    my ($Shop,$TaxMatrix) = @_;

    # correct defaults for taxareadigital and taxclassdigital on Shop
    # Attributes must point to system tax matrix digital
    if($Shop->get('DefaultTaxAreaDigital')) {
        # fix error for switching to default after brexit UNITY-6876
        if (!$TaxMatrix->existsChild($Shop->get('DefaultTaxAreaDigital')->alias) && $Shop->get('DefaultTaxAreaDigital')->alias eq 'UnitedKingdom') {
            $Shop->set({'DefaultTaxAreaDigital' => $TaxMatrix->child('UnitedKingdomEU')});
        } else {
            $Shop->set({'DefaultTaxAreaDigital' => $TaxMatrix->child($Shop->get('DefaultTaxAreaDigital')->alias)});
        };
    } else {
        # use systems default
        $Shop->set({'DefaultTaxAreaDigital' => $TaxMatrix->get('DefaultTaxAreaDigital')});
    }
    if($Shop->get('DefaultTaxClassDigital')) {
        $Shop->set({'DefaultTaxClassDigital' => $TaxMatrix->child($Shop->get('DefaultTaxClassDigital')->alias)});
    } else {
        # use systems default
        $Shop->set({'DefaultTaxClassDigital' => $TaxMatrix->get('DefaultTaxClassDigital')});
    }
}

sub NewTaxAreaAdded {
    my $self = shift;
    my ($SystemTaxMatrix, $Matrix) = @_;

    my %HelperHash = map { $_->alias,1 } @{$SystemTaxMatrix->get('TaxAreas')};
    foreach my $aTaxArea (@{$Matrix->get('TaxAreas')}) {
        if(!exists $HelperHash{$aTaxArea->alias}) {
            return 1;
        }
    }

    return 0;
}

sub NewTaxClassAdded {
    my $self = shift;
    my ($SystemTaxMatrix, $Matrix) = @_;

    my %HelperHash = map { $_->alias,1 } @{$SystemTaxMatrix->get('TaxClasses')};
    foreach my $aTaxClass (@{$Matrix->get('TaxClasses')}) {
        if(!exists $HelperHash{$aTaxClass->alias}) {
            return 1;
        }
    }

    return 0;
}

sub SaveShopCountries {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object;

    if ($Servlet->params('NewCountryID')){
        #add a new country to the shopcountries with the given position
        my @hCountries;
        my $Position = $Servlet->params('Position');
        $Position = POSITION_DEFAULT if $Position !~ /\d+/; # set value for position if not set or charachter used (add to last position in list)
        if ($Servlet->params('NewCountryID') eq 'all') {
            foreach my $Country (@{$Shop->get('UnassignedCountries')}) {
                push @hCountries, {'Position' => $Position, 'ID' => $Country->{'CountryID'}};
                $Position++;
            }
        } else {
            push @hCountries, {'Position' => $Position, 'ID' => $Servlet->params('NewCountryID')};
        }
        foreach my $Country (@hCountries) {
            $Shop->addCountry($Country->{ID},$Country->{Position},0) unless $Shop->existsCountry($Country->{ID});
        }
        $Shop->renumberCountries();
        return $Servlet->vars('CountryAdded',1);
    }

    # save positions for existing countries
    my $DefaultID = $Servlet->form->value('IsDefault');
    my $hForm = $Servlet->form->form($Shop, 'ListedCountries');
    foreach my $hItem( @{$hForm->{'ListCountries'}} ) {
        my $CountryID = $hItem->{'CountryID'};
        my $Position = $hItem->{'ListPosition'};
        my $IsDefault = (defined $DefaultID and $CountryID == $DefaultID) ? 1:0;
        $Shop->updateCountry($CountryID,$Position,$IsDefault);
    }


    #save sort order for shopcountries
    return $Shop->renumberCountries();

}

sub DeleteShopCountries {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object;
    my @CountryIDs = $Servlet->form->value('ListObjectID');

    #save sort order for shopcountries
    foreach my $CountryID (@CountryIDs) {
        $Shop->removeCountry($CountryID);
    }
    return $Shop->renumberCountries();

}

sub UnityDeleteShopCountries {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object;
    my @CountryIDs = $Servlet->form->value('ListObjectID');

    # do nothing if no valid country selected
    return if !scalar @CountryIDs;

    # delete only if accepted if confirmation is used
    if ($Servlet->user->get('DeleteConfirmation') and not $Servlet->params('ACCEPT')) {
        # set delete mark to template
        my @DeleteObjects;
        foreach my $Item (@CountryIDs) {
            push @DeleteObjects, GetInfoCountry($Item);
        }
        $Servlet->vars('DeleteObjects',\@DeleteObjects);
        $Servlet->vars('DELETE',1);
        return;
    } else {
        # else reset delete
        $Servlet->vars('DELETE',0);
    }

    foreach my $CountryID (@CountryIDs) {
        $Shop->removeCountry($CountryID);
    }
    return $Shop->renumberCountries();

}


sub DeleteTaxMethod {
    my $self = shift;
    my ($Servlet) = @_;
    my $ObjectID = $Servlet->form->value('ListObjectID');
    return if ! $ObjectID;

    my $Shop = $Servlet->object;

    if ($Servlet->user->get('DeleteConfirmation') and not $Servlet->params('ACCEPT')) {
        my %ShopTaxTypeMap = map { $_->get('TaxType')->id => $_ } @{$Shop->get('TaxMethods')};
        # map taxtypes to taxmethods
        $Servlet->vars('DeleteObjects', [ map { $ShopTaxTypeMap{$_}->tleHash } grep { defined $ShopTaxTypeMap{$_} } ($ObjectID) ]);
        $Servlet->vars('DELETE',1);
        return;
    }

    # disable method
    my $CurrentTaxMethod = $Shop->get('TaxMethod');
    DisableShopTaxMethod($Shop) if defined $CurrentTaxMethod and $CurrentTaxMethod->id eq $ObjectID;

    $self->DeleteListedObjects($Servlet, LoadClassByAlias('TaxMethod'));
    return;
}

sub SetShopTaxMethod {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object;

    my @DefaultValues = $Servlet->form->value('IsDefault');
    return unless scalar @DefaultValues == 1;

    #test if we have to create the method first.
    my $TaxTypeID = shift @DefaultValues;
    my $TaxType = LoadRootObject()->loadObject($TaxTypeID, 'TaxType');
    my $TaxMethod;
    foreach my $Method (@{$Shop->get('TaxMethods')}) {
        next if $TaxTypeID != $Method->get('TaxType')->id;
        $TaxMethod = $Method;
    }

    if(not defined $TaxMethod){
        # create TaxMethod
        $Servlet->params('ListObjectID', $TaxTypeID);
        $self->_CreateTaxMethod($Servlet);
    }

    # activate TaxMethod for Shop
    $TaxMethod = $TaxMethod // $Shop->folder('TaxMethods')->child($TaxType->alias);
    return unless ($TaxMethod->instanceOf('TaxMethod') && $TaxMethod->getSite()->id eq $Shop->id);

    eval {
        EnableShopTaxMethod($Shop, $TaxMethod);
    };
    if (ExistsError()) {
        my $Error = GetError();
        if ($Error->code ~~ [qw( TaxMatrixNotCustomized TaxMethodNotUsableAtStorefront )]) {
            $Servlet->vars('DEFAULT_NOT_POSSIBLE',1);
        } else {
            Error($Error);
        }
    }

    $Servlet->vars('ViewAction', 'MBO-ViewTaxTypes');
    TriggerHook('UI_AfterSetShopTaxMethod', {'Servlet' => $Servlet, 'TaxMethod' => $TaxMethod});
}

# create shop tax method from the TaxType
sub _CreateTaxMethod {
    my $self = shift;
    my ($Servlet) = @_;

    return if ! $Servlet->params('ListObjectID');
    my $CreateTaxTypeID = $Servlet->params('ListObjectID');

    return unless $CreateTaxTypeID;

    my $Shop = $Servlet->object;
    # test if Method with the given name exists
    my $Folder = GetTaxMethodsFolder($Shop);

    foreach my $TaxMethod (@{$Shop->get('TaxMethods')}){
        my $ShopTaxTypeID = $TaxMethod->get('TaxType')->id;
        # prevent creation of taxmethods of the same tax type.
        return if ($ShopTaxTypeID eq $CreateTaxTypeID);
    }

    eval {
        my $TaxType = LoadRootObject()->loadObject($CreateTaxTypeID, 'TaxType');
        TestObject('TaxType',$TaxType,'DE_EPAGES::Tax::API::Object::TaxType');
        $TaxType->newTaxMethod($Shop, {
            'Alias' => $TaxType->alias,
            'Parent' => $Folder,
        });
    };
    if (ExistsError()) {
        my $Error = GetError();
        Error($Error) unless($Error->code eq 'FeatureLimitExceeded');
        ResetError();
        $Servlet->vars('FeatureMaxValueReached', $Error->vars);
    }

    return;
}

sub ViewTaxTypes {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object;

    my $TaxMethod = $Shop->get('TaxMethod');

    # choose the correct ViewAction if we have TaxUS because we use an own template set
    if(defined $TaxMethod and ExistsClassByAlias('TaxMethodTaxUS') and $TaxMethod->instanceOf('TaxMethodTaxUS')) {
        unless($Servlet->params('ViewAction') =~ /MBO-ViewTaxTypesUS/g ){
            $Servlet->vars('ViewAction', 'MBO-ViewTaxTypesUS');
            $Servlet->doViewAction;
        }
    }

    return $self->SUPER::View($Servlet);
}

sub ConfigureTaxMethod {
     my $self = shift;
     my ($Servlet) = @_;
     my $Shop = $Servlet->object;

     # enable and activate taxmethod on shop
     # by getting the taxtype, creating the method, if not exists
     # and activating the method for the shop

    #test if we have to create the method first.
    my $ConfigureTaxTypeID = $Servlet->params('ListObjectID');
    my $TaxType = LoadRootObject()->loadObject($ConfigureTaxTypeID, 'TaxType');
    my $TaxMethod;
    foreach my $Method (@{$Shop->get('TaxMethods')}) {
        next if $ConfigureTaxTypeID != $Method->get('TaxType')->id;
        $TaxMethod = $Method;
    }

    if(not defined $TaxMethod){
        # create TaxMethod
        $Servlet->params('ListObjectID', $ConfigureTaxTypeID);
        $self->_CreateTaxMethod($Servlet);
    }

    # configure TaxMethod for Shop
    $TaxMethod = $TaxMethod // $Shop->folder('TaxMethods')->child($TaxType->alias);

    $Servlet->vars('ViewAction', 'MBO-ViewGeneral');
    $Servlet->vars('ViewObjectID', $TaxMethod->id);

}

sub UnityViewTaxCalculation {
    my $self = shift;
    my ($Servlet) = @_;

    # enable language switch
    $Servlet->vars('ShowLanguageSwitch', 1);

    $self->SUPER::View($Servlet);
}

sub SaveTaxModel {
    my $self = shift;
    my ($Servlet) = @_;

    my $Form = $Servlet->form;
    $Form->form($Servlet->object, 'TaxModel');

    my $Index = 0;
    foreach my $VATreleasedMessage ($Form->value('VATreleasedMessage')) {
        # throw an error in case an empty (or whitespace) VATreleasedMessage has been entered (UNITY-5496)
        $Form->addFormError({
            'Reason'   => 'EMPTY_VAT_RELEASED_MESSAGE',
            'Name'     => 'VATreleasedMessage',
            'LoopName' => 'VATreleasedMessage',
            'Index'    => $Index,
        }) if ($Form->value('VATreleased') && (!defined $VATreleasedMessage || $VATreleasedMessage !~ /\S/ ));
        $Index++;
    }

    $self->SUPER::Save($Servlet);
}

1;
